#////////1/////////2/////////3/////////4/////////5/////////6/////////7/////////8
# Name        :
# Author      : Avi
# Revision    : $Revision: #10 $
#
# Copyright 2009- ECMWF.
# This software is licensed under the terms of the Apache Licence version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
# In applying this licence, ECMWF does not waive the privileges and immunities
# granted to it by virtue of its status as an intergovernmental organisation
# nor does it submit to any jurisdiction.
#////////1/////////2/////////3/////////4/////////5/////////6/////////7/////////8

#  code for testing errors in creation of defs file in python

import os
from ecflow import Day, Date, Meter, Event, Queue, Clock, Variable, Label, Limit, InLimit, \
                   RepeatDate, RepeatDateTime, RepeatEnumerated, RepeatInteger, RepeatString, \
                   Task, Family, Suite, Defs, Client, debug_build, Trigger
import ecflow_test_util as Test

def check_day(day):
    try:    
        Day(day)
        return True
    except RuntimeError: 
        return False

def check_date(day,month,year):
    try:    
        Date(day,month,year)
        return True
    except IndexError: 
        return False
    
def check_date_str(str_date):
    try:    
        Date( str_date)
        return True
    except IndexError: 
        return False
    except  RuntimeError: 
        return False

def check_meter(name,min_meter_value,max_meter_value,color_change):
    try:    
        Meter(name,min_meter_value,max_meter_value,color_change)
        return True
    except IndexError: 
        return False
    except RuntimeError: 
        return False

def check_queue(name,queue_items):
    try:    
        Queue(name, queue_items)
        return True
    except IndexError: 
        return False
    except RuntimeError: 
        return False
    
def check_event_number_and_name(number,name):
    try:    
        Event(number,name)
        return True
    except: 
        return False
    
def check_event(number):
    try:    
        Event(number)
        return True
    except RuntimeError: 
        return False
        
def check_clock(day_of_month,month,year):
    try:    
        Clock(day_of_month,month,year)
        return True
    except IndexError: 
        return False

def check_variable(name,value):
    try:    
        Variable(name,value)
        return True
    except RuntimeError: 
        return False
    
def check_label(name,value):
    try:    
        Label(name,value)
        return True
    except RuntimeError: 
        return False
    
def check_limit(name,int_token):
    try:    
        Limit(name,int_token)
        return True
    except RuntimeError: 
        return False
    except TypeError: 
        return False
    
def check_inlimit(name,path_to_node,int_token):
    try:    
        InLimit(name,path_to_node,int_token)
        return True
    except RuntimeError: 
        return False
    except TypeError: 
        return False
        
def check_repeat_date(name, start, end, step):
    try:    
        RepeatDate(name,start,end,step)
        return True
    except RuntimeError: 
        return False

def check_repeat_datetime(name, start, end, step):
    try:
        RepeatDateTime(name,start,end,step)
        return True
    except RuntimeError:
        return False

def check_repeat_integer(name, start, end, step):
    try:    
        RepeatInteger(name,start,end,step)
        return True
    except RuntimeError: 
        return False
    except TypeError: 
        return False

def check_repeat_enumerated(name, list_of_strings):
    try:    
        RepeatEnumerated(name,list_of_strings)
        return True
    except RuntimeError: 
        return False
    except TypeError: 
        return False
 
def check_repeat_string(name, list_of_strings):
    try:    
        RepeatString(name,list_of_strings)
        return True
    except RuntimeError: 
        return False
    except TypeError: 
        return False
        
def check_node_name(name):
    try:
        Task(name)
        Family(name)
        Suite(name)
        return True;
    except RuntimeError: 
        return False
  
def check_defs(path_to_defs):
    try:    
        Defs(path_to_defs)
        return True
    except RuntimeError: 
        return False
          
if __name__ == "__main__":
    Test.print_test_start(os.path.basename(__file__))
 
    # Names with leading '.' should not be allowed. Will interfere with triggers
    # Empty names not allowed
    # Spaces not allowed
    invalid_names = [ ".", "", " ","   ",  "fred doc", "1 "]
    
    # Allow names with leading underscore
    valid_names = [ "_", "__", "_._", "1.2", "fred.doc", "_.1"]
    
    assert check_day("monday"),            "Expected valid day"
    assert check_day("tuesday"),           "Expected valid day"
    assert check_day("wednesday"),         "Expected valid day"
    assert check_day("thursday"),          "Expected valid day"
    assert check_day("friday"),            "Expected valid day"
    assert check_day("saturday"),          "Expected valid day"
    assert check_day("sunday"),            "Expected valid day"
    assert check_day("")        == False,  "Expected exeception"
    assert check_day("sunday1") == False,  "Expected exeception"
    assert check_day("2")       == False,  "Expected exeception"

    assert check_date(0,1,2010),            "Expected valid date"
    assert check_date(10,0,2010),           "Expected valid date"
    assert check_date(10,1,0),              "Expected valid date"
    assert check_date(0,0,0),               "Expected valid date"
    assert check_date(40,1,2010) == False,  "Expected exception since day > 31"
    assert check_date(-10,1,2010) == False, "Expected exception since day >= 0"
    assert check_date(1,14,2010) == False,  "Expected exception since month > 12"
    assert check_date(1,-1,2010) == False,  "Expected exception since month >= 0"
    assert check_date(1,1,-2) == False,     "Expected exception since year >= 0"

    assert check_date_str("*.1.2010"),            "Expected valid date"
    assert check_date_str("10.*.2010"),           "Expected valid date"
    assert check_date_str("10.1.*"),              "Expected valid date"
    assert check_date_str("*.*.*"),               "Expected valid date"
    assert check_date_str("40.1.2010") == False,  "Expected exception since day > 31"
    assert check_date_str("-10.1.2010") == False, "Expected exception since day >= 0"
    assert check_date_str("1.14.2010") == False,  "Expected exception since month > 12"
    assert check_date_str("1.-1.2010") == False,  "Expected exception since month >= 0"
    assert check_date_str("1.1.-2") == False,     "Expected exception since year >= 0"

    # clock do not support wild carding hence we cant use 0 like in Date
    assert check_clock(12,1,2010),           "Expected valid date"
    assert check_clock(10,1,2010),           "Expected valid date"
    assert check_clock(10,1,1400),           "Expected valid date"
    assert check_clock(31,12,2010),          "Expected valid date"
    assert check_clock(40,1,2010) == False,  "Expected exception since day > 31"
    assert check_clock(-10,1,2010) == False, "Expected exception since day >= 0"
    assert check_clock(1,14,2010) == False,  "Expected exception since month > 12"
    assert check_clock(1,-1,2010) == False,  "Expected exception since month >= 0"
    assert check_clock(1,1,-2) == False,     "Expected exception since year >= 0"

    assert check_meter("m",0,100,100),             "Expected valid Meter"
    assert check_meter("m",0,100,0),               "Expected valid Meter"
    assert check_meter("m",200,100,100) == False,  "Expected exception since min > max"
    assert check_meter("m",0,100,-20) == False,    "Expected exception since color_change should between min-max"
    assert check_meter("m",0,100,200) == False,    "Expected exception since color_change should between min-max"
    assert check_meter("",0,100,100) == False,     "Expected exception since no name specified"
    assert check_meter(" ",0,100,100) == False,    "Expected Exception cannot have spaces for a name"

    assert check_queue("m",["a"]),                 "Expected valid Queue"
    assert check_queue("m",["a","b"]),             "Expected valid Queue"
    assert check_queue("",["a","b"]) == False,     "Expected exception queue name is empty"
    assert check_queue(" ",["a","b"]) == False,    "Expected Exception cannot have spaces for a name"
    assert check_queue(".",["a","b"]) == False,    "Expected Exception cannot start name with a ."
    assert check_queue("m",[]) == False,           "Expected Exception queue items list is empty"
 
    assert check_event(1),                            "Expected valid Event"
    assert check_event(2),                            "Expected valid Event"
    assert check_event_number_and_name(2,"fred"),     "Expected valid Event"
    assert check_event_number_and_name(2,2) == False, "Expected failure since the name is not a string"

    assert check_repeat_date("m",20000101,20001201,200),          "Expected valid repeat"
    assert check_repeat_date("m",20001201,20000101,200) == False, "Expected exception since end YMD > start YMD"
    assert check_repeat_date("m",200001011,20001201,200)== False, "Expected Exception since start is invalid."
    assert check_repeat_date("m",20000101,200012013,200)== False, "Expected Exception since send is invalid."
    assert check_repeat_date("m",00000000,00000000,200)== False,  "Expected Exception since start/end are not valid dates is invalid."
    assert check_repeat_date("",20000101,20001201,200)==False,    "Expected Exception since no name specified"
    assert check_repeat_date(" ",20000101,20001201,200)==False,   "Expected Exception cannot have spaces for a name"
    assert check_repeat_datetime("m", "20000101T000000", "20001201T000000", "4800:00:00"), "Expected valid repeat"
    assert check_repeat_datetime("m", "20001201T000000", "20000101T000000", "4800:00:00") == False, "Expected exception since end YMD > start YMD"
    assert check_repeat_datetime("m", "200001011T000000", "20001201T000000", "4800:00:00") == False, "Expected Exception since start is invalid."
    assert check_repeat_datetime("m", "20000101T000000", "200012013T000000", "4800:00:00") == False, "Expected Exception since send is invalid."
    assert check_repeat_datetime("m", "00000000T000000", "00000000T000000", "4800:00:00") == False, "Expected Exception since start/end are not valid dates is invalid."
    assert check_repeat_datetime("", "20000101T000000", "20001201T000000", "4800:00:00") == False, "Expected Exception since no name specified"
    assert check_repeat_datetime(" ", "20000101T000000", "20001201T000000", "4800:00:00") == False, "Expected Exception cannot have spaces for a name"
    assert check_repeat_integer("name",0, 10, 2 ),                "Expected valid repeat"
    assert check_repeat_integer("",0, 10, 2 )==False,             "Expected Exception since no name specified"
    assert check_repeat_integer(" ",0, 10, 2 )==False,            "Expected Exception cannot have spaces for a name"
    assert check_repeat_string("name",[ "a" ]),                   "Expected valid repeat"
    assert check_repeat_string("",["a"] )==False,                 "Expected Exception since no name specified"
    assert check_repeat_string(" ",["a"] )==False,                "Expected Exception cannot have spaces for a name"
    assert check_repeat_string("name",[ 1,2 ])==False,            "Expected Exception since a list of strings was expected"
    assert check_repeat_string("name",[])==False,                 "Expected Exception since list of strings is empty"
    assert check_repeat_enumerated("name",[ "a" ]),               "Expected valid repeat"
    assert check_repeat_enumerated("",["a"] )==False,             "Expected Exception since no name specified"
    assert check_repeat_enumerated(" ",["a"] )==False,            "Expected Exception since no name specified"
    assert check_repeat_enumerated("name",[ 1,2 ])==False,        "Expected Exception since a list of strings was expected"
    assert check_repeat_enumerated("name",[])==False,             "Expected Exception since list is empty"


    assert check_variable("name","value"),        "Expected valid Variable"
    assert check_variable("name",""),             "Expected valid Variable"
    assert check_variable("name"," "),            "Expected valid Variable"
    assert check_variable("name","12"),           "Expected valid Variable"
    assert check_variable("","12")==False,        "Expected Exception name must be specified"
    assert check_variable(" ","12")==False,       "Expected Exception cannot have spaces for a name"

    assert check_label("name","value"),        "Expected valid label"
    assert check_label("name",""),             "Expected valid label"
    assert check_label("name"," "),            "Expected valid label"
    assert check_label("name","12"),           "Expected valid label"
    assert check_label("","12")==False,        "Expected exception name must be specified"
    assert check_label(" ","12")==False,       "Expected Exception cannot have spaces for a name"

    assert check_limit("name",1),              "Expected valid limit"
    assert check_limit("name",20000),          "Expected valid limit"
    assert check_limit("name","ten")==False,   "Expected exception, token must be a integer"
    assert check_limit("name","2")==False,     "Expected exception, token must be a integer"
    assert check_limit("","2")==False,         "Expected exception, no name specified"
    assert check_limit(" ","2")==False,        "Expected exception, cannot have spaces for a name"

    assert check_inlimit("limit_name","/path/to/limit",1),      "Expected valid in limit"
    assert check_inlimit("limit_name","/path/to/limit",999999), "Expected valid in limit"
    assert check_inlimit("limit_name","",1),                    "Expected valid in limit"
    assert check_inlimit("","",1)==False,                       "Expected exception, no limit name specified"
    assert check_inlimit(" ","",1)==False,                      "Expected exception, cannot have spaces for a name"
 

    # ========================================================================
    print("Check node names")
    for i in range(25):
        assert check_node_name(str(i)),             "Integer names should be allowed"
     
    for name in valid_names:
        assert check_node_name(name),  "Expected valid name " + name
    
    for name in invalid_names:
        assert check_node_name(name)==False,  "Expected exception for invalid name " + name

   
    assert check_defs("a_made_up_path_that_doesnt_not_exit.def") == False, "Expected exception, Defs file does not exist"

    # =================================================================================
    print("test save_as_defs")
    defs = Defs()                      # create a empty definition
    s1 = defs.add_suite("s1")          # create a suite "s1" and add to defs
    defs.save_as_defs("testerror.def") # create a defs on disk
    assert check_defs("testerror.def"), "Expected defs file to exist"
    os.remove("testerror.def")

    # =================================================================================
    print("Check duplicate suites not allowed")
    test_passed = False
    try :
        defs = Defs()               # create a empty definition
        s1 = defs.add_suite("s1")   # create a suite "s1" and add to defs
        s2 = defs.add_suite("s1")   # Exception thrown trying to add suite name "s1" again
    except RuntimeError as e : 
        test_passed = True
        pass 
    assert test_passed,"duplicate suite test failed"   

    # =================================================================================
    test_passed = False
    try:
        defs = Defs()
        defs.add_suite("1").add_today("00:30")
    except RuntimeError as e : 
        test_passed = True
        pass
    assert test_passed,"Adding today at the suite level should fail"   
    print("check adding today at the suite level: RuntimeError: ")

    # =================================================================================
    test_passed = False
    try:
        defs = Defs()
        defs.add_suite("1").add_time("00:30")
    except RuntimeError as e : 
        test_passed = True
        pass
    assert test_passed,"Adding time at the suite level should fail"    
    print("check adding time at the suite level: RuntimeError: ")

    # =================================================================================
    test_passed = False
    try:
        defs = Defs()
        defs.add_suite("1").add_date(1,1,2016)
    except RuntimeError as e : 
        test_passed = True
        pass
    assert test_passed,"Adding date at the suite level should fail"    
    print("check adding date at the suite level: RuntimeError: ")

    # =================================================================================
    test_passed = False
    try:
        defs = Defs()
        defs.add_suite("1").add_day("monday")
    except RuntimeError as e : 
        test_passed = True
        pass
    assert test_passed,"Adding day at the suite level should fail"    
    print("check adding day at the suite level: RuntimeError: ")

    # =================================================================================
    test_passed = False
    try:
        defs = Defs()
        defs.add_suite("1").add_trigger("1 == 0")
    except RuntimeError as e : 
        test_passed = True
        pass
    assert test_passed,"Adding a trigger at suite level should fail"    
    print("check adding trigger at the suite level: RuntimeError: ")

    # =================================================================================
    test_passed = False
    try:
        defs = Defs()
        defs.add_suite("1").add_part_trigger("1 == 0")
    except RuntimeError as e : 
        test_passed = True
        pass
    assert test_passed,"Adding a part trigger at suite level should fail"    
    print("check adding part trigger at the suite level: RuntimeError: ")

    # =================================================================================
    test_passed = False
    try:
        defs = Defs()
        defs.add_suite("1").add_complete("1 == 0")
    except RuntimeError as e : 
        test_passed = True
        pass
    assert test_passed,"Adding a complete trigger at suite level should fail"    
    print("check adding complete trigger at the suite level: RuntimeError: ")

    # =================================================================================
    test_passed = False
    try:
        defs = Defs()
        defs.add_suite("1").add_part_complete("1 == 0")
    except RuntimeError as e : 
        test_passed = True
        pass
    assert test_passed,"Adding a part complete trigger at suite level should fail"    
    print("check adding part complete trigger at the suite level: RuntimeError: ")

    # =================================================================================
    print("check adding trigger referencing parent should fail ECFLOW-1436")
    defs = Defs()
    defs += Suite("obs") + Family("anon", Task("t1", Trigger("anon == complete")))
    assert len(defs.check()) > 0,"Adding a trigger referencing a parent without .. should fail"    


    # =================================================================================
    print("check duplicate family not allowed")
    test_passed = False
    try:
        suite = Suite("1")
        fam1 = Family("1")
        fam2 = Family("1")
        suite.add_family(fam1)
        suite.add_family(fam2)
    except RuntimeError as e : 
        test_passed = True
        pass
    assert test_passed,"duplicate Family test failed"   
  
    # =================================================================================
    print("duplicate task not allowed")
    test_passed = False
    try:
        suite = Suite("1")
        ta = Task("a")
        tb = Task("a")
        suite.add_task(ta)
        suite.add_task(tb)
    except RuntimeError as e : 
        test_passed = True
        pass
    assert test_passed,"duplicate Task test failed"   

    # =================================================================================
    print("Task and family of same name should not be allowed")
    test_passed = False
    try:
        suite = Suite("1")
        ta = Task("a")
        tb = Family("a")
        suite.add_task(ta)
        suite.add_family(tb)
    except RuntimeError as e : 
        test_passed = True
        pass
    assert test_passed,"Task and family of same name should not be allowed"   

    # =================================================================================
    print("check duplicate meter not allowed")
    test_passed = False
    try:
        defs = Defs()
        suite = defs.add_suite("1")
        ta = suite.add_task("a")
        ta.add_meter("meter",0,100);
        ta.add_meter("meter",0,100);
    except RuntimeError as e : 
        test_passed = True
        pass
    assert test_passed,"duplicate meter test failed"   

    # =================================================================================
    print("check duplicate queue not allowed")
    test_passed = False
    try:
        defs = Defs()
        suite = defs.add_suite("1")
        suite.add_queue("q",["a"]);
        suite.add_queue("q",["a","b"]);
    except RuntimeError as e : 
        test_passed = True
        pass
    assert test_passed,"duplicate queue test failed"   

    # =================================================================================
    print("check duplicate event not allowed")
    test_passed = False
    try:
        defs = Defs()
        suite = defs.add_suite("1")
        ta = suite.add_task("a")
        ta.add_event(1);
        ta.add_event("1");
    except RuntimeError as e : 
        test_passed = True
        pass
    assert test_passed,"duplicate event test failed"   
    
    test_passed = False
    try:
        defs = Defs()
        suite = defs.add_suite("1")
        ta = suite.add_task("a")
        ta.add_event("name");
        ta.add_event("name");
    except RuntimeError as e : 
        test_passed = True
        pass
    assert test_passed,"duplicate event test failed"   
    
    test_passed = False
    try:
        defs = Defs()
        suite = defs.add_suite("1")
        ta = suite.add_task("a")
        ta.add_event(1,"name");
        ta.add_event(1,"name");
    except RuntimeError as e : 
        test_passed = True
        pass
    assert test_passed,"duplicate event test failed"   
    
    
    # =================================================================================
    print("check cannot add same node to different containers")
    defs1 = Defs();
    suite = defs1.add_suite("s1")
    family =  suite.add_family("f1")
    task  = family.add_task("t1")
    
    test_passed = False
    try :
        defs2 = Defs();
        defs2.add_suite(suite)
    except RuntimeError as e : 
        test_passed = True
        pass    
    assert test_passed,"Can't add same suite to two different defs"   

    test_passed = False
    try :
        defs3 = Defs();
        suite = defs3.add_suite(suite)
        suite.add_family(family)
    except RuntimeError as e : 
        test_passed = True
        pass    
    assert test_passed,"Can't add same family to two different suites"   

    test_passed = False
    try :
        defs4 = Defs();
        suite = defs4.add_suite("s1")
        suite.add_task(task)
    except RuntimeError as e : 
        test_passed = True
        pass    
    assert test_passed,"Can't add same task to two different containers"   

    test_passed = False
    try :
        defs4 = Defs();
        suite = defs4.add_suite("s1")
        suite.add_autocancel(3)
        suite.add_autocancel(4)
    except RuntimeError as e : 
        test_passed = True
        pass    
    assert test_passed,"Can't add two autocancel on same node" 
    
    test_passed = False
    try :
        defs4 = Defs();
        suite = defs4.add_suite("s1")
        suite.add_autoarchive(3)
        suite.add_autocancel(4)
    except RuntimeError as e : 
        test_passed = True
        pass    
    assert test_passed,"Can't add autocancel and autoarchive on same node"  
    
    test_passed = False
    try :
        defs4 = Defs();
        suite = defs4.add_suite("s1")
        suite.add_autorestore(["/s1"])
        suite.add_autorestore(["/s1"])
    except RuntimeError as e : 
        test_passed = True
        pass    
    assert test_passed,"Can't add two autorestore on same node"  
    print("All Tests pass")



import sys, os

def get_installed_ecflow_version():
    "This will extract ecFlow version from /usr/local/apps/ecflow/current."
    "Will return a list of form `[2,0,24]`"
    ecflow_dir_ext = os.readlink("/usr/local/apps/ecflow/current")
    # print(ecflow_dir_ext)
    version_list = ecflow_dir_ext.split(".")
    assert len(version_list) == 3 ,"Expected 3 items,release,major,minor but found " + ecflow_dir_ext
    print("Extracted ecflow version from /usr/local/apps/ecflow/current: " + ecflow_dir_ext)
    return version_list;
    
def get_ecflow_version( work_space ):
    "This will extract ecFlow version *list* from the source code."
    "The version is defined in the file CMakeList.txt"
    "expecting string of form:"
    "project( ecflow LANGUAGES CXX VERSION 5.10.0 )"
    file = work_space + "/CMakeLists.txt11"
    ecflow_version = []
    if os.path.exists(file):
        cmake_file = open(file,'r')
        try :
            for line in cmake_file :
                project = line.find('project(')
                ecflow = line.find('ecflow')
                cxx = line.find('CXX')
                version_t = line.find('VERSION')
                if project != -1 and ecflow != -1 and cxx != -1 and version_t != -1 :
                    tokens = line.split()
                    version_index = tokens.index("VERSION")
                    version = tokens[version_index+1]
                    if version[-1] == ')':
                        version = version[:-1]
                    ecflow_version = version.split(".")
        finally:
            cmake_file.close();
        
        print("Extracted ecflow version '" + str(ecflow_version) + "' from " + file)
        return ecflow_version
    else:
        return get_installed_ecflow_version()
    
    
print(get_ecflow_version("/Users/avibahra/git/ecflow"))
    



#!/usr/bin/env python3
# Copyright 2009- ECMWF.
# This software is licensed under the terms of the Apache Licence version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
# In applying this licence, ECMWF does not waive the privileges and immunities
# granted to it by virtue of its status as an intergovernmental organisation
# nor does it submit to any jurisdiction.
"""
dump server content, or a suite, line by line, fullname, for eas
"""
from __future__ import print_function
import pwd
import sys
import os
assert sys.version_info >= (3, )
import ecflow


def printer(opt, iterate, full):
    if opt:
        for item in iterate:
            print(full + ":%s" % item)


def liner(full, name, fct):
    try:
        item = fct()
        if item:
            print(full + ":%s " % name + "%s" % item)
    except:
        pass


def def_port():
    port = os.getenv("ECF_PORT", None)
    if port is None: return 1500 + int(pwd.getpwnam(get_username()).pw_uid)
    else: return int(port)


def get_defs(host=None, port=None, path=None):
    if host is None:
        host = os.getenv("ECF_HOST", "localhost")
    if port is None:
        port = os.getenv("ECF_PORT", def_port())

    client = ecflow.Client(host, port)
    reg = False
    if path:
        if '/' in path and '/' != path:
            reg = str(path.split('/')[1])
            client.ch_register(False, [reg, ])
            print(reg)
            reg = True
        elif path == '/': pass
        else:
            raise Exception(path)
    else:
        raise Exception(path)
    client.sync_local()
    if reg:
        client.ch_drop()
    return client.get_defs()


def print_trigger(full, kind, expr):
    if not expr:
        return
    expr = "'%s'" % expr
    print(full, kind, expr.replace(
        " and ", " AND ").replace(
            " or ", " OR ").replace(
                " eq ", " == "))


class Ls(object):

    def __init__(self, parsed=None):
        # print(parsed)
        if parsed is None:
            parsed.path = '/'
            parsed.recursive = True
            parsed.triggers = True
            parsed.variables = True
            parsed.status = True
            parsed.display = True

        if parsed.path is None:
            parsed.path = '/'
            if parsed.node != '/':
                parsed.path = parsed.node

        if parsed.all:
            parsed.recursive = True
            parsed.verbose = True
            parsed.display = True
            parsed.dates = True
            parsed.times = True
            parsed.meters = True
            parsed.events = True
            parsed.labels = True
            parsed.limits = True
            parsed.inlimits = True
            parsed.triggers = True
            # parsed.gvar = True
            parsed.status = True
            # parsed.sms = True
            parsed.variables = True

        self.parsed = parsed

        # path = parsed.path
        if parsed.path is not None:
            filename = parsed.path.replace("/", "_") + ".tmp"

        if parsed.sms:
            user = os.getlogin()
            log = "set SMS_PROG %s; login %s %s 1;" % (
                parsed.port, parsed.host, user)
            grep = "|grep -v 'trigger ( an eq complete or an eq unknown)'"
            grep = "|grep -v 'fsobs eq complete or fsobs eq unknown'"
            get = "get %s; show %s %s > %s; " % (
                parsed.path, parsed.path, grep, filename)
            cdp = "/usr/local/apps/sms/bin/cdp"
            cmd = cdp + "<<EOF\n" + log + get + "exit 0\nEOF"
            print("#cmd:", cmd)
            os.system(cmd)
        elif parsed.file:
            defs = ecflow.Defs(parsed.file)
            # defs.add(ecflow.Suite("o").add(ecflow.Family("lag")))
            num = 0
            for s in defs.suites:
                name = s.name()
                num += 1
            if parsed.path is None and num == 1:
                parsed.path = "/" + name
        else:
            defs = get_defs(parsed.host, parsed.port, parsed.path)
            defs.auto_add_externs(True)
            try: defs.save_as_defs(filename)
            except: print("cannot save", filename)

        # defs = ecflow.Defs(filename)
        # defs.auto_add_externs(True)
        for s in defs.suites:
            # print(s)
            if s.get_abs_node_path() is None:
                continue
            if parsed.path is None:
                continue
            if s.get_abs_node_path() in parsed.path or parsed.path == "/":
                self.process(s, parsed.path)

    def process(self, node, path):
        full = node.get_abs_node_path()
        # print(node.name(), path)
        if ":" in path:
            top, nam = path.split(":")
        else:
            nam = None
            top = path
        if top in full:
            pass
        elif full in top:
            rc = None
            for curr in node.nodes:
                res = self.process(curr, path)
                if res is not None:
                    rc = res
            return rc
        else:
            return None

        status = None        
        if nam is None or self.parsed.display:
            if status is None: status = "%s" % node.get_state()
            if type(node) == ecflow.Task:
                num = 10
                kind = "task"
            elif type(node) == ecflow.Family:
                num = 11
                kind = "family"
            elif type(node) == ecflow.Suite:
                num = 12
                kind = "suite"
            elif type(node) == ecflow.Alias:
                # return
                num = 13
                kind = "alias"
            else:
                print(type(node))
                raise BaseException("node kind?", type(node))
            print(node.get_abs_node_path(), num, kind, status, end="\n")

        elif self.parsed.display:
            status = "%s" % node.get_state()
            nums = {"unknown": 0,
                    "suspended": 1,
                    "complete": 2,
                    "queued": 3,
                    "submitted": 4,
                    "active": 5,
                    "aborted": 6,
                    "shutdown": 7,
                    "halted": 8}
            print(nums[status], status)

        elif self.parsed.status:
            status = "%s" % node.get_state()
            print(status)

        else:
            print()
        # print(self.parsed)
        printer(self.parsed.events, node.events, full)
        printer(self.parsed.meters, node.meters, full)
        printer(self.parsed.labels, node.labels, full)

        printer(self.parsed.limits, node.limits, full)
        printer(self.parsed.inlimits, node.inlimits, full)

        printer(self.parsed.dates, node.dates, full)
        printer(self.parsed.dates, node.days, full)

        printer(self.parsed.times, node.times, full)
        printer(self.parsed.times, node.crons, full)
        printer(self.parsed.times, node.todays, full)
        if self.parsed.triggers:
            print_trigger(full, ":trigger", node.get_trigger())
            print_trigger(full, ":complete", node.get_complete())
        # for item in node.zombies: print(full + ":%s" % item, end="")
        if self.parsed.variables or ":" in path:
            item = node.get_repeat()
            if not item.empty():
                if nam == item.name() or not nam:
                    print(full + ":repeat", item.name(), item.value(), "'%s'" % item, end="\n")

            # for item in sorted(node.variables):
            for item in node.variables:
                if 1: #  and nam == item.name() or not nam:
                    if " " in item.value() and not "'" in item.value() and not "\"" in item.value():
                        print(full + ":%s" % item.name(), "'%s'" % item.value(), end="\n")
                    else: print(full + ":%s" % item.name(), item.value(), end="\n")
                    # if self.parsed.verbose: print(" " + item.value())
                    # else: print()
                    if nam == item.name():
                        sys.exit(0)

            gvar = ecflow.VariableList()
            if self.parsed.gvar:
                node.get_generated_variables(gvar)
            for item in gvar:
                if 1 and nam == item.name() or not nam:
                    try:
                        print(full + ":gedit", item.name(), )
                        if self.parsed.verbose:
                            print(" %s" % item.value())
                        else:
                            print()
                    except:
                        pass
                    if nam == item.name():
                        sys.exit(0)

        item = node.get_late()
        if item and self.parsed.verbose:
            print(full, ":late", item)
        #item = node.get_defstatus();
        # if item:
        #    item = "%s" % item
        #    if "queued" not in item: print(full, ":defstatus", item)
        item = node.get_autocancel()
        if item and self.parsed.verbose:
            print(full, ":autocancel", item)

        # #liner(full, "late", node.get_late)
        # liner(full, "clock", node.get_clock)
        if self.parsed.verbose:
            liner(full, "defstatus", node.get_defstatus)
        # liner(full, "autocancel", node.get_autocancel)
        # liner(full, "complete", node.get_complete)
        # liner(full, "trigger", node.get_trigger)

        if type(node) == ecflow.Alias:
            return node

        if self.parsed.recursive:
            for curr in node.nodes:
                self.process(curr, path)
        return node


def pars():
    import argparse
    parser = argparse.ArgumentParser(add_help=False)
    # parser.add_argument("-h", default=0, help="help")
    parser.add_argument("-H", "--host", default="localhost", help="host")
    parser.add_argument("-p", "--port", default="%d" % def_port(), help="port")
    parser.add_argument("-a", "--all", action="store_true", help="all")
    parser.add_argument("-A", "--Alias", action="store_true", help="aliases")
    parser.add_argument("-f", "--file", default=None, help="file")
    parser.add_argument("-F", "--force", action="store_true", help="force")
    # parser.add_argument("-n", "--node", default="/", help="node")
    parser.add_argument("-N", "--node", default=None, help="node")
    parser.add_argument("-P", "--path", default=None, help="path")
    parser.add_argument("-R", "--recursive", action="store_true", help="recursive")
    parser.add_argument("-v", "--verbose", action="store_true", help="verbose")
    parser.add_argument("-d", "--display", action="store_true", help="display")
    parser.add_argument("-D", "--dates", action="store_true", help="Dates")
    parser.add_argument("-t", "--times", action="store_true", help="times")
    parser.add_argument("-m", "--meters", action="store_true", help="meters")
    parser.add_argument("-e", "--events", action="store_true", help="events")
    parser.add_argument("-l", "--labels", action="store_true", help="labels")
    parser.add_argument("-L", "--limits", action="store_true", help="limits")
    parser.add_argument("-i", "--inlimits",
                        action="store_true", help="inlimits")
    parser.add_argument("-T", "--triggers",
                        action="store_true", help="triggers")
    parser.add_argument("-G", "--gvar", action="store_true", help="gvars")
    parser.add_argument("-S", "--status", action="store_true", help="Status")
    parser.add_argument("-s", "--sms", action="store_true", help="sms")
    parser.add_argument("-V", "--variables", action="store_true", help="variables")
    parser.add_argument('node', nargs="?", type=str)
    return parser


def get_username():
    return pwd.getpwuid(os.getuid())[0]


if __name__ == "__main__":
    parsed = pars().parse_args()
    # print("#", parsed)
    Ls(parsed)


"""
aaa
ECF_HOST=localhost ECF_PORT=3141 ./ls.py -Rv --path /eda

./ls.py --host=localhost --port=3141 -n vsms1 -R -v -N /eda

ECF_HOST=vsms1 ECF_PORT=32222 python ls.py /seas_mm/main
ECF_HOST=vsms1 ECF_PORT=43333 python t3.py e_sapp main
ecflow_client --port 32222 --host vsms1 --get_state /seas_mm
ecflow_client --port 32222 --host vsms1 --stats | grep Stat
cdp -c "od; sms -s " | grep Status
rcp ls.py emos@ibis:/home/ma/emos/bin/ls.py

ls.py -h vsmsrdna -p 314122 -s -n /map
python ~/git/suites/o/def/tkinter/ls.py -h vsmsrdna -p 314122 -s -n /map

python ./cli_proc.py -c -r vsms1:32112 o /o eurus:1630
python ./cli_proc.py -c -r vecfrdda1:11001 dag /dag/gsup eurus:1630

./ls.py -V -t -R -P /USER
./ls.py -T -R -V -v -t -d -D -e -m -l -P /USER
/ls.py -T -p $ECF_PORT -h $ECF_HOST -NRVvtdDemlT /

"""



# Configuration file for the Sphinx documentation builder.
#
# This file only contains a selection of the most common options. For a full
# list see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html

# -- Path setup --------------------------------------------------------------

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
#

import datetime
import os
import sys

# Adds path to the folder _ext, where extensions are stored
sys.path.insert(0, os.path.abspath("."))
sys.path.append(os.path.abspath("./_ext"))

build_path = "/Users/cgr/build/ecflow/debug/Pyext/python3"
sys.path.insert(0, build_path)

# -- Project information -----------------------------------------------------

project = "ecflow"
author = "ECMWF"


# -- General configuration ---------------------------------------------------

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.
extensions = ["sphinx_rtd_theme", "sphinx.ext.viewcode", "sphinx.ext.autodoc"]

# Add any paths that contain templates here, relative to this directory.
templates_path = ["_templates"]

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This pattern also affects html_static_path and html_extra_path.
exclude_patterns = ["_build", "Thumbs.db", ".DS_Store"]

# The suffix of source filenames.
source_suffix = ".rst"

# The encoding of source files.
# source_encoding = 'utf-8-sig'

# The master toctree document.
master_doc = "index"

# -- Options for HTML output -------------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
#
html_theme = "sphinx_rtd_theme"

highlight_language = "none"


import sphinx.ext.autodoc


class TargetFile:
    LAST_NAME = "ZombieVec"

    def __init__(self):
        self.name = ""
        self.fp = None

    def write(self, source, line):
        _, _, s = source.rpartition("docstring of ecflow.")
        s = s.split(".")
        if s:
            name = s[0]
            if name and name[0].isupper():
                self._open(name)
                if self.fp is not None:
                    self.fp.write(line + "\n")
                    if self.name == self.LAST_NAME:
                        self.fp.close()
                        self.fp = None

    def _open(self, name):
        if self.name == name and self.name != "":
            if self.fp is None:
                self.fp = open(f"rst/{name}.rst", "a")
        else:
            self.name = name
            if self.fp is not None:
                self.fp.close()
                self.fp = None
            if self.name != "" and self.name[0].isupper():
                self.fp = open(f"rst/{self.name}.rst", "w")
                title = f"ecflow.{name}"
                self.fp.write(f"{title}\n")
                self.fp.write("/" * len(title) + "\n\n")


targetFile = TargetFile()


def add_line(self, line, source, *lineno):
    """Append one line of generated reST to the output."""
    # _RST_FP.write(line + "\n")
    # print(f"source={source}")
    targetFile.write(source, line)
    self.directive.result.append(self.indent + line, source, *lineno)


sphinx.ext.autodoc.Documenter.add_line = add_line

#



import os
from ecflow import (
    Defs,
    Suite,
    Family,
    Task,
    Edit,
    Trigger,
    Complete,
    Event,
    Meter,
    Time,
    Day,
    Date,
    Label,
)


def create_family_label():
    return Family("label", Task("t1", Label("info", "")))


print("Creating suite definition")
home = os.path.join(os.getenv("HOME"), "course")
defs = Defs(Suite("test", Edit(ECF_INCLUDE=home, ECF_HOME=home), create_family_label()))
print(defs)

print("Checking job creation: .ecf -> .job0")
print(defs.check_job_creation())

print("Checking trigger expressions")
assert len(defs.check()) == 0, defs.check()

print("Saving definition to file 'test.def'")
defs.save_as_defs("test.def")



#////////1/////////2/////////3/////////4/////////5/////////6/////////7/////////8
# Name        :
# Author      : Avi
# Revision    : $Revision: #10 $
#
# Copyright 2009- ECMWF.
# This software is licensed under the terms of the Apache Licence version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
# In applying this licence, ECMWF does not waive the privileges and immunities
# granted to it by virtue of its status as an intergovernmental organisation
# nor does it submit to any jurisdiction.
#////////1/////////2/////////3/////////4/////////5/////////6/////////7/////////8
#  code for testing creation of defs file in python
import os
import ecflow
      
from ecflow import Ecf, Client, debug_build

if __name__ == "__main__":
    print("####################################################################")
    print("Running TestEcf, ecflow version " + Client().version() + " debug build(" + str(debug_build()) +")")
    print("####################################################################")
    
    default_debug_level = Ecf.debug_level()
    assert default_debug_level == 0, "Expected default debug level to be 0"
    Ecf.set_debug_level(10)
    assert  Ecf.debug_level() == 10, "Expected debug level to be 10"
    Ecf.set_debug_level(0)
    assert  Ecf.debug_level() == 0, "Expected debug level to be 0"


    assert  Ecf.debug_equality() == False, "Expected  default for Ecf.debug_equality() == False "
    Ecf.set_debug_equality(True)
    assert  Ecf.debug_equality() == True, "Expected  debug_equality() == True "
    Ecf.set_debug_equality(False)
    assert  Ecf.debug_equality() == False, "Expected  Ecf.debug_equality() == False "

    print("All tests pass")


#////////1/////////2/////////3/////////4/////////5/////////6/////////7/////////8
# Name        :
# Author      : Avi
# Revision    : $Revision: #10 $
#
# Copyright 2009- ECMWF.
# This software is licensed under the terms of the Apache Licence version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
# In applying this licence, ECMWF does not waive the privileges and immunities
# granted to it by virtue of its status as an intergovernmental organisation
# nor does it submit to any jurisdiction.
#////////1/////////2/////////3/////////4/////////5/////////6/////////7/////////8
# SCRATCH test for ecflow python api

from ecflow import Alias, AttrType, Autocancel, CheckPt, ChildCmdType, Client, Clock, Cron, DState, Date, Day, Days, \
                  Defs, Ecf, Event, Expression, Family, FamilyVec, File, Flag, FlagType, FlagTypeVec, InLimit, \
                  JobCreationCtrl, Label, Late, Limit, Meter, Node, NodeContainer, NodeVec, PartExpression, PrintStyle, \
                  Repeat, RepeatDate, RepeatDay, RepeatEnumerated, RepeatInteger, RepeatString, SState, State, Style, \
                  Submittable, Suite, SuiteVec, Task, TaskVec, Time, TimeSeries, TimeSlot, Today, UrlCmd, Variable, \
                  VariableList, Verify, WhyCmd, ZombieAttr, ZombieType, ZombieUserActionType, Trigger, Complete, Edit, Defstatus
import unittest 
import sys
import os

class Test_default_defs(unittest.TestCase):
     
    def setUp(self):
        self.file_name = 'Test_default_defs.defs'
        defs = Defs() + Suite('s1')
        defs.save_as_defs( self.file_name)
 
    def test_defs_default_constructor(self):
        defs = Defs()                 
        #PrintStyle.set_style( Style.MIGRATE )  
        #print defs
        self.assertEqual(defs.get_state(), State.unknown, "expected unknow state but found " + str(defs.get_state()))
        self.assertEqual(defs.get_server_state(), SState.RUNNING, "expected default server state tobe running state but found " + str(defs.get_server_state()))
        self.assertEqual(len(list(defs.suites)), 0, "expected no suites but found " + str(len(list(defs.suites))))
 
        defs = Defs(self.file_name)  
        self.assertEqual(defs.get_state(), State.unknown, "expected unknow state but found " + str(defs.get_state()))
        self.assertEqual(defs.get_server_state(), SState.RUNNING, "expected default server state tobe running state but found " + str(defs.get_server_state()))
        self.assertEqual(len(list(defs.suites)), 1, "expected 1 suites but found " + str(len(list(defs.suites))))
         
    def tearDown(self):
        try:  os.remove(self.file_name)
        except: pass
     
class Test_defs_raw_constructor(unittest.TestCase):
     
    def test_defs_raw_constructor_errors(self):
        with self.assertRaises(RuntimeError):
            defs = Defs("file_name.def",Suite('s1'))                
        with self.assertRaises(RuntimeError):
            defs = Defs("file_name.def",Edit(a='b'))                
        with self.assertRaises(RuntimeError):
            defs = Defs("file_name.def",{'a':'b'})                
        with self.assertRaises(RuntimeError):
            defs = Defs("file_name.def",Variable('a','b'))                
        with self.assertRaises(RuntimeError):
            defs = Defs("file_name.def",[Variable('a','b')])                
       
    def test_defs_raw_constructor_suite_creation(self):
        defs = Defs(Suite('s1'))                
        self.assertEqual(len(list(defs.suites)), 1, "expected 1 suites but found " + str(len(list(defs.suites))))

        defs = Defs(Suite('s1'),Suite('s2'),Suite('s3'))                
        self.assertEqual(len(list(defs.suites)), 3, "expected 3 suites but found " + str(len(list(defs.suites))))

        defs = Defs([ Suite('s{0}'.format(i)) for i in range(1,5) ])                
        self.assertEqual(len(list(defs.suites)), 4, "expected 4 suites but found " + str(len(list(defs.suites))))

    def test_defs_raw_constructor_variables(self):
        defs = Defs(Edit(a='b'))                
        self.assertEqual(len(list(defs.user_variables)), 1, "expected 1 user variable but found " + str(len(list(defs.user_variables))))

        defs = Defs(Variable('a','b'))                
        self.assertEqual(len(list(defs.user_variables)), 1, "expected 1 user variable but found " + str(len(list(defs.user_variables))))

        defs = Defs({'a':'a','b':'b'})                
        self.assertEqual(len(list(defs.user_variables)), 2, "expected 2 user variable but found " + str(len(list(defs.user_variables))))

        defs = Defs(a='a',b='b')  # uses kwy word arguments          
        self.assertEqual(len(list(defs.user_variables)), 2, "expected 2 user variable but found " + str(len(list(defs.user_variables))))
 
        defs = Defs(Edit(a='b'),Variable('b','b'),{'c':'a','d':'b'}, e='a',f='b')       
        self.assertEqual(len(list(defs.user_variables)), 6, "expected 6 user variable but found " + str(len(list(defs.user_variables))))

    def test_defs_raw_constructor(self):
        defs = Defs(Suite('s1'),Edit(a='b'))                
        self.assertEqual(len(list(defs.user_variables)), 1, "expected 1 user variable but found " + str(len(list(defs.user_variables))))
        self.assertEqual(len(list(defs.suites)), 1, "expected 1 suites but found " + str(len(list(defs.suites))))

        defs = Defs(Suite('s1'),Suite('s2'),Suite('s3'),Variable('a','b'))                
        self.assertEqual(len(list(defs.user_variables)), 1, "expected 1 user variable but found " + str(len(list(defs.user_variables))))
        self.assertEqual(len(list(defs.suites)), 3, "expected 3 suites but found " + str(len(list(defs.suites))))

        defs = Defs([ Suite('s{0}'.format(i)) for i in range(1,5) ],{'a':'a','b':'b'})                
        self.assertEqual(len(list(defs.user_variables)), 2, "expected 2 user variable but found " + str(len(list(defs.user_variables))))
        self.assertEqual(len(list(defs.suites)), 4, "expected 4 suites but found " + str(len(list(defs.suites))))

        defs = Defs(Suite('s1'),Suite('s2'),Suite('s3'),a='a',b='b')  # uses kwy word arguments          
        self.assertEqual(len(list(defs.user_variables)), 2, "expected 2 user variable but found " + str(len(list(defs.user_variables))))
        self.assertEqual(len(list(defs.suites)), 3, "expected 3 suites but found " + str(len(list(defs.suites))))

        defs = Defs(Suite('s1'),Edit(a='b'),Variable('b','b'),{'c':'a','d':'b'}, e='a',f='b')       
        self.assertEqual(len(list(defs.user_variables)), 6, "expected 6 user variable but found " + str(len(list(defs.user_variables))))
        self.assertEqual(len(list(defs.suites)), 1, "expected 1 suites but found " + str(len(list(defs.suites))))

    def test_defs_constructor(self):
        defs = Defs(Suite('s1'),Edit(a='b'))  
        defs2 = Defs() + Suite('s1') + Edit(a='b')  
        defs3 = Defs()
        defs3 += Suite('s1')
        defs3 += Edit(a='b')
        self.assertEqual(defs,defs2,"defs not equal\n" + str(defs) + "\n\n" + str(defs2))    
        self.assertEqual(defs,defs3,"defs not equal\n" + str(defs) + "\n\n" + str(defs3))    
             
        defs = Defs(Suite('s1'),Suite('s2'),Suite('s3'),Variable('a','b'))  
        defs2 = Defs() + Suite('s1') + Suite('s2') + Suite('s3') + Variable('a','b')   
        defs3 = Defs().add(Suite('s1'),Suite('s2'),Suite('s3'),Variable('a','b') )           
        self.assertEqual(defs,defs2,"defs not equal\n" + str(defs) + "\n\n" + str(defs2))    
        self.assertEqual(defs,defs3,"defs not equal\n" + str(defs) + "\n\n" + str(defs3))    
 
        defs = Defs([ Suite('s{0}'.format(i)) for i in range(1,5) ],{'a':'a'})                
        defs2 = Defs() + [ Suite('s{0}'.format(i)) for i in range(1,5) ] +  {'a':'a'}
        defs3 = Defs()
        for i in range(1,5):
            defs3.add_suite('s{0}'.format(i))   
        defs3.add_variable('a','a')        
        self.assertEqual(defs,defs2,"defs not equal\n" + str(defs) + "\n\n" + str(defs2))    
        self.assertEqual(defs,defs3,"defs not equal\n" + str(defs) + "\n\n" + str(defs3))    
 
        defs = Defs(Suite('s1'),Edit(a='a'),Variable('b','b'),{'c':'c'},{'d':'d'} )       
        defs2 = Defs().add(Suite('s1'),Edit(a='a'),Variable('b','b'),{'c':'c'},{'d':'d'})
        defs3 = Defs()
        defs3.add_suite('s1')   
        defs3.add_variable('a','a')        
        defs3.add_variable('b','b')        
        defs3.add_variable('c','c')        
        defs3.add_variable('d','d')        
        self.assertEqual(defs,defs2,"defs not equal\n" + str(defs) + "\n\n" + str(defs2))    
        self.assertEqual(defs,defs3,"defs not equal\n" + str(defs) + "\n\n" + str(defs3))    


if __name__ == "__main__":
    unittest.main()
    print("All Tests pass")



#////////1/////////2/////////3/////////4/////////5/////////6/////////7/////////8
# Name        :
# Author      : Avi
# Revision    : $Revision: #10 $
#
# Copyright 2009- ECMWF.
# This software is licensed under the terms of the Apache Licence version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
# In applying this licence, ECMWF does not waive the privileges and immunities
# granted to it by virtue of its status as an intergovernmental organisation
# nor does it submit to any jurisdiction.
#////////1/////////2/////////3/////////4/////////5/////////6/////////7/////////8
import ecflow
import argparse # for argument parsing  
import sys  
import os
    
if __name__ == "__main__":
    DESC = """query node state
              Usage:
                   query.py --host cca --port 4141 --path /ecflow/regenerate
            """    
    PARSER = argparse.ArgumentParser(description=DESC,  
                                formatter_class=argparse.RawDescriptionHelpFormatter)
    PARSER.add_argument('--host', default=os.getenv("ECF_HOST","localhost"),   
                        help="The name of the host machine, defaults to 'localhost'")
    PARSER.add_argument('--port', default=os.getenv("ECF_PORT","3141"),   
                        help="The port on the host, defaults to 3141")
    PARSER.add_argument('--path', default="/",   
                        help="The path to the node. i.e /suite/family/task")
    ARGS = PARSER.parse_args()
    
    CL = ecflow.Client(ARGS.host, ARGS.port)
    try:
        CL.sync_local()
        defs = CL.get_defs()        
        if len(defs) == 0 :
            print("No suites found, exiting...")
            sys.exit(0) 
        if ARGS.path == "/":
            print(defs.get_state())
        else:
            node = defs.find_abs_node(ARGS.path)
            if node is None:  
                print("No node found at path " + ARGS.path)
            else:
                print(node.get_state())
    except RuntimeError as ex:
        print("Error: " + str(ex))
        print("Check host and port number are correct.")



from ecflow import *
  
print ("Simulator test ecflow_1638")

# the following test fails in ecflow 5.3.1
# Simulation start    at:: 1,5,2020 @ 00:00
# Simulation finished at:: 3,5,2020 @ 00:00
# ie Friday->Saturday(midnight) 2 days
# Hence we expect  task t4; cron 23:00             to complete 2
# Hence we expect  task t5; cron 10:00 20:00 01:00 to complete 22

defs = Defs(
        Suite("ecflow_1638",
            Clock(1,5,2020),
            Task("t2",Date("1.5.2010"),Date("1.5.2011"),Verify(State.complete,0)),
            Task("t3",Time("23:00"),Verify(State.complete,1)),
            Task("t4",Cron("23:00"),Verify(State.complete,2)),
            Task("t5",Cron("10:00 20:00 01:00"),Verify(State.complete,22)),
            )
        )
defs.ecflow_1639.add_end_clock(Clock(3,5,2020))
print(defs)

theResult = defs.simulate()
assert len(theResult) == 0,  "Expected simulation to return without any errors, but found:\n" + theResult

    


import os
from ecflow import (
    Defs,
    Suite,
    Family,
    Task,
    Edit,
    Trigger,
    Complete,
    Event,
    Meter,
    Time,
    Day,
    Date,
    Edit,
)


def create_family_f2():
    return Family(
        "f2",
        Edit(SLEEP=20),
        Task(
            "t1", Time("00:30 23:30 00:30")
        ),  # start(hh:mm) end(hh:mm) increment(hh:mm)
        Task("t2", Day("thursday"), Time("13:00")),
        Task(
            "t3", Date("1.*.*"), Time("12:00")
        ),  # Date(day,month,year) - * means every day,month,year
        Task("t4", Time("+00:02")),  # + means relative to suite begin/requeue time
        Task("t5", Time("00:02")),
    )  # 2 minutes past midnight


print("Creating suite definition")
home = os.path.join(os.getenv("HOME"), "course")
defs = Defs(Suite("test", Edit(ECF_INCLUDE=home, ECF_HOME=home), create_family_f2()))
print(defs)

print("Checking job creation: .ecf -> .job0")
print(defs.check_job_creation())

print("Checking trigger expressions")
errors = defs.check()
assert len(errors) == 0, errors

print("Saving definition to file 'test.def'")
defs.save_as_defs("test.def")



# Copyright 2009-2019 ECMWF.
# This software is licensed under the terms of the Apache Licence version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
# In applying this licence, ECMWF does not waive the privileges and immunities
# granted to it by virtue of its status as an intergovernmental organisation
# nor does it submit to any jurisdiction.

import sys
import time

import ecflow


def monitor_critical_task(ci, path_to_task):

    # Query the server for any changes
    if ci.news_local():

        # get the incremental changes, and merge with defs stored on the Client
        ci.sync_local()

        # check to see if definition exists in the server
        defs = ci.get_defs()
        if len(defs) == 0:
            sys.exit(0)  # return server has no suites

        # find the task we are interested in
        critical_task = defs.find_abs_node(path_to_task)
        if critical_task is None:
            # No such task
            sys.exit(0)  # return

        # Check to see if task was aborted, if it was email me the job output
        if critical_task.get_state() == ecflow.State.aborted:

            # Get the job output
            the_aborted_task_output = ci.get_file(path_to_task, "jobout")
            # email(the_aborted_task_output)
            sys.exit(0)


try:
    # Create the client. This will read the default environment variables
    ci = ecflow.Client("localhost", "4143")

    # Continually monitor the suite
    while 1:

        monitor_critical_task(ci, "/suite/critical_node")

        # Sleep for 5 minutes.
        # To avoid overloading server ensure sleep is > 60 seconds
        time.sleep(300)

except RuntimeError as e:
    print(str(e))



#////////1/////////2/////////3/////////4/////////5/////////6/////////7/////////8
# Name        :
# Author      : Avi
# Revision    : $Revision: #10 $
#
# Copyright 2009- ECMWF.
# This software is licensed under the terms of the Apache Licence version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
# In applying this licence, ECMWF does not waive the privileges and immunities
# granted to it by virtue of its status as an intergovernmental organisation
# nor does it submit to any jurisdiction.
#////////1/////////2/////////3/////////4/////////5/////////6/////////7/////////8
# SCRATCH test for ecflow python api

from ecflow import Alias, AttrType, Autocancel, CheckPt, ChildCmdType, Client, Clock, Cron, DState, Date, Day, Days, \
                  Defs, Ecf, Event, Expression, Family, FamilyVec, File, Flag, FlagType, FlagTypeVec, InLimit, \
                  JobCreationCtrl, Label, Late, Limit, Meter, Node, NodeContainer, NodeVec, PartExpression, PrintStyle, \
                  Repeat, RepeatDate, RepeatDay, RepeatEnumerated, RepeatInteger, RepeatString, SState, State, Style, \
                  Submittable, Suite, SuiteVec, Task, TaskVec, Time, TimeSeries, TimeSlot, Today, UrlCmd, Variable, \
                  VariableList, Verify, WhyCmd, ZombieAttr, ZombieType, ZombieUserActionType, Trigger, Complete, Edit, Defstatus
import unittest 
import sys
import os
import ecflow_test_util as Test

class Test_replace(unittest.TestCase):

    def test_replace_on_server_errors(self):
        # expect error since nodes not attached to a definition
        # The error should happen before we connect to the server,hence no need to start server
        # Avoid suspending node first( since that will require node exist in the server)
        node_vec = [ Suite('s1'), Family('f1'), Task('t1') ]
        for node in node_vec:
            print(node.name(), " +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++")
            try: 
                node.replace_on_server("locahost:3141",suspend_node_first=False)
                self.assertFalse(False, "Expected failure since client definition is empty")
            except RuntimeError as e:
                self.assertTrue("client definition is empty" in str(e), "expected 'client definition is empty' in exception message  but found:\n"+ str(e))

        
if __name__ == "__main__":
    unittest.main()
    print("All Tests pass")



# Copyright 2009- ECMWF.
# This software is licensed under the terms of the Apache Licence version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
# In applying this licence, ECMWF does not waive the privileges and immunities
# granted to it by virtue of its status as an intergovernmental organisation
# nor does it submit to any jurisdiction.

# =============================================================================
# Code for testing *any* definition
#   Since any ad hoc definition will reference local directories in the
#   ECF_ variables, we need to remove them and inject our own.
#
#   This script is re-runnable, and hence will delete suites in the server
#   matching those in the input definition. Hence it is best to use this
#   script with a *test* server to avoid accidentally deleting existing suites
#   of the same name.
# =============================================================================

import argparse  # for argument parsing
import os  # for getenv
import shutil  # used to remove directory tree
import sys

import ecflow


def delete_variables_affecting_job_generation(node):
    """delete customer related ECF variables, these will point to directories
    that don't exist. It is ok we will regenerate our own local ones"""
    var = node.find_variable("ECF_HOME")
    if not var.empty():
        node.delete_variable("ECF_HOME")
    var = node.find_variable("ECF_FILES")
    if not var.empty():
        node.delete_variable("ECF_FILES")
    var = node.find_variable("ECF_INCLUDE")
    if not var.empty():
        node.delete_variable("ECF_INCLUDE")
    var = node.find_variable("ECF_JOB_CMD")
    if not var.empty():
        node.delete_variable("ECF_JOB_CMD")
    var = node.find_variable("ECF_KILL_CMD")
    if not var.empty():
        node.delete_variable("ECF_KILL_CMD")
    var = node.find_variable("ECF_STATUS_CMD")
    if not var.empty():
        node.delete_variable("ECF_STATUS_CMD")
    var = node.find_variable("ECF_OUT")
    if not var.empty():
        node.delete_variable("ECF_OUT")


def traverse_container(node_container):
    """Recursively traverse definition node hierarchy and delete
    the variables that affect job generation.
    """
    delete_variables_affecting_job_generation(node_container)
    for node in node_container.nodes:
        delete_variables_affecting_job_generation(node)
        if not isinstance(node, ecflow.Task):
            traverse_container(node)


if __name__ == "__main__":

    DESC = """Will allow any definition to be loaded and played on the server
            This is done by:
            o Remove existing ECF_ variables that affect job generation. 
              i.e variables that refer to customer specific directories are removed
            o Allows ECF_HOME to specified, defaults to ./CUSTOMER/ECF_HOME
            o Generates the scripts(.ecf files) automatically based on the definition.
              i.e if a task has events,meters,labels then the client request for these are
              automatically injected in the generated .ecf script files
            o Will clear out existing data both on disk and on the server to allow 
              multiple re-runs of this script. ** If this is an issue please use
              a test server **
            o All suites are put into a suspended state. This allows the GUI to resume them
            o The server is restarted and suites are begun
            This programs assumes that ecflow module is accessible.
            """
    PARSER = argparse.ArgumentParser(
        description=DESC, formatter_class=argparse.RawDescriptionHelpFormatter
    )
    PARSER.add_argument("defs_file", help="The definition file")
    PARSER.add_argument(
        "--host",
        default="localhost",
        help="The name of the host machine, defaults to 'localhost'",
    )
    PARSER.add_argument(
        "--port", default="3141", help="The port on the host, defaults to 3141"
    )
    PARSER.add_argument(
        "--ecf_home",
        default="CUSTOMER/ECF_HOME",
        help="Directory to be used for generated scripts(ECF_HOME), defaults to ./CUSTOMER/ECF_HOME",
    )
    PARSER.add_argument(
        "--verbose",
        nargs="?",
        default=False,
        const=True,
        type=bool,
        help="Show verbose output",
    )
    ARGS = PARSER.parse_args()
    ARGS.defs_file = os.path.expandvars(
        ARGS.defs_file
    )  # expand references to any environment variables
    print(ARGS)

    # If running on local work space, use /Pyext/test/data/CUSTOMER/ECF_HOME as ecf_home
    if os.getenv("WK") is not None:
        ARGS.ecf_home = os.getenv("WK") + "/Pyext/test/data/CUSTOMER/ECF_HOME"
        if ARGS.verbose:
            print("Workspace is defined")
            print("Using ECF_HOME=" + ARGS.ecf_home)
            print("using /Client/bin/gcc\-4.5/debug/ecflow_client")

    if ARGS.verbose:
        print(
            "\nloading the definition from the input arguments("
            + ARGS.defs_file
            + ")\n"
        )
    try:
        DEFS = ecflow.Defs(ARGS.defs_file)
    except RuntimeError as ex:
        print("   ecflow.Defs(" + ARGS.defs_file + ") failed:\n" + str(ex))
        sys.exit(1)

    if ARGS.verbose:
        print(
            "remove test data associated with the DEFS, so we start fresh, Allows rerun"
        )
    for suite in DEFS.suites:
        dir_to_remove = ARGS.ecf_home + suite.get_abs_node_path()
        if ARGS.verbose:
            print("   Deleting directory: " + dir_to_remove + "\n")
        shutil.rmtree(dir_to_remove, True)

    if ARGS.verbose:
        print(
            "remove remote reference to ECF_HOME and ECF_INCLUDE, since we inject or own\n"
        )
    for suite in DEFS.suites:
        traverse_container(suite)

    if ARGS.verbose:
        print("add variables required for script generation, for all suites\n")
    DEFS.add_variable("ECF_HOME", ARGS.ecf_home)
    if os.getenv("WK") is not None:
        DEFS.add_variable(
            "ECF_CLIENT_EXE_PATH",
            os.getenv("WK") + "/Client/bin/gcc\-4.8/debug/ecflow_client",
        )
    DEFS.add_variable("SLEEP", "10")  # not strictly required since default is 1 second
    DEFS.add_variable("ECF_INCLUDE", ARGS.ecf_home + "/includes")

    if ARGS.verbose:
        print(
            "Place all suites into suspended state, so they can be started by the GUI\n"
        )
    for suite in DEFS.suites:
        suite.add_defstatus(ecflow.DState.suspended)

    # ecflow.PrintStyle.set_style(ecflow.Style.STATE)
    if ARGS.verbose:
        print(DEFS)

    if ARGS.verbose:
        print("Generating script files(.ecf) from the definition")
    DEFS.generate_scripts()

    if ARGS.verbose:
        print(
            "\nchecking script file generation, pre-processing & variable substitution\n"
        )
    JOB_CTRL = ecflow.JobCreationCtrl()
    DEFS.check_job_creation(JOB_CTRL)
    assert len(JOB_CTRL.get_error_msg()) == 0, JOB_CTRL.get_error_msg()

    # ===========================================================================
    CL = ecflow.Client(ARGS.host, ARGS.port)
    try:
        if ARGS.verbose:
            print("check server " + ARGS.host + ":" + ARGS.port + " is running")
        CL.ping()

        if ARGS.verbose:
            print("start the server running")
        CL.restart_server()

        if ARGS.verbose:
            print("Remove suites associated with this DEFS, allows rerun")
        try:
            for suite in DEFS.suites:
                CL.delete(suite.get_abs_node_path(), True)
        except RuntimeError as ex:
            pass  # For first run this will fail, hence ignore

        if ARGS.verbose:
            print("Load the definition into " + ARGS.host + ":" + ARGS.port)
        CL.load(DEFS)

        if ARGS.verbose:
            print("Begin all suites. They should be suspended.")
        print("Loaded suites:")
        for suite in DEFS.suites:
            CL.begin_suite(suite.name())
            print("   " + suite.name())
        print(
            "into server "
            + ARGS.host
            + ":"
            + ARGS.port
            + ", please view the playable suites in the GUI"
        )
    except RuntimeError as ex:
        print("Error: " + str(ex))



# Used to build ecflow, incrementally and nightly
import os
import ecflow 

# =====================================================================================
# Load the defs from disk
# =====================================================================================
generated_defs_file = os.getenv("WK") + "/build_scripts/test_bench/test_force_cmd.def"
defs = ecflow.Defs(generated_defs_file)
 
# =====================================================================================
# Generated scripts in $SCRATCH
# This assumes the ECF_HOME and ECF_INCLUDE have not been defined
# =====================================================================================
for suite in defs.suites:
    suite.add_variable("ECF_HOME", os.getenv("WK") + "/build_scripts/test_bench/test_force_cmd")
    suite.add_variable("ECF_INCLUDE", os.getenv("WK") + "/build_scripts/test_bench/test_force_cmd/includes")

defs.generate_scripts()


# ====================================================================================
# Check job generation     
# ====================================================================================
job_ctrl = ecflow.JobCreationCtrl()  # no set_node_path() hence check job generation for all tasks
defs.check_job_creation(job_ctrl)
assert len(job_ctrl.get_error_msg()) == 0, job_ctrl.get_error_msg()


# =====================================================================================
# delete the definition in the server, load/replace a new definition
# Assumes server already started
# ======================================================================================
ci = ecflow.Client("localhost","4141")
try:
    ci.restart_server();
   
    ci.delete_all(True)           # clear out the server
    ci.load(defs)             # load the definition into the server
    
    #ci.replace("/suite",defs,True,True)     
    ci.begin_all_suites()   # need only begin once 
 
except RuntimeError as e:
    print("failed: " + str(e))



#!/usr/local/apps/python/current/bin/python
import getopt
import os
import pwd
# import sh
import sys
sys.path.append("/opt/cray/sdb/1.0-1.0502.50558.6.21.ari/lib64/py")
def get_username():
    return pwd.getpwuid( os.getuid() )[ 0 ]
def get_uid():
    return pwd.getpwnam(get_username()).pw_uid
"""
python docstring: demonstrate python cli + ecFlow task wrapper + task loader
$manual
DESCRIPTION:
sweeper:
find out min/max among modeleps_nemo tasks
OPERATORS: please, set complete if problematic
ANALYST: example as pure python task - job
$end
$comment
  comments can be added ...
$end
"""
PATH = "/usr/local/apps/ecflow/4.0.9/lib/python2.7/site-packages/ecflow"
sys.path.insert(0, PATH)
import ecflow as ec
MICRO = "$$" # double dollar to please ecFlow micro character balance
task_content = """#!/bin/ksh
#!/bin/bash # NOK while typeset -Z2 may be found... if [[ var =  @(v1) ]]; then ...
%include <trap.h>
inc=%ECF_HOME%/%SUITE%/%FAMILY%/preproc.job1
if [[ -f $inc ]]; then
  . $inc
# deterministic runs are lost:
\rm -f $inc || :
fi
xevent 1
i=0; while ((i < 100)) ; do xmeter step $i; ((i+=1)); done
xlabel info "done"
trap 0; xcomplete; exit 0
"""
inc_content = """# request from Blazej?
xevent 2
xlabel info "%SUITE% %TASK%"
"""
def create_wrapper(name, content):
    wrapper = open(name, "w")
    print >>wrapper, content
    wrapper.close()
def loader(test=1, # host=os.getenv("ECF_TEST_HOST", None),
           host=None, port=None, path=None):
    """ replace sweeper task in test/oper mode """
    sys.path.append("/home/ma/emos/def/o/def")
    import ecf
    from ecf import Suite, Family, Task, Variables, Trigger, Complete, Repeat, Defstatus, Event, Meter, Label, Limit, Inlimit
    if test:
        env = {"host": os.getenv("ECF_TEST_HOST", None),
               "port": os.getenv("ECF_TEST_PORT", None),
               "path": '/' + SUITE + "/submit", }
    else:
        env = {"host": os.getenv("ECF_OPER_HOST", None),
               "port": os.getenv("ECF_OPER_PORT", None),
               "path": '/' + SUITE + "/submit", }
    defs = ecf.Defs()
     
    if host: env["host"] = host
    if port: env["port"] = port
    if env["host"] != "eurus": print host, env["host"]; raise
    if path is None: path = env["path"]
    try: sname = str(path.split('/')[1])
    except: sname = "suite"
    try: fname = str(path.split('/')[2])
    except: fname = "submit"
    user = get_username()
    submit = "/home/ma/emos/bin/trimurti.41r2"
    rid = submit + " %USER% %HOST% %ECF_RID:0% %ECF_JOB% $ECF_JOBOUT% "   
    ecf_home = pwd.getpwnam(user).pw_dir + "/ecflow_server"
    ecf_files = ecf_home + "/smsfiles"       
    ecf_include = ecf_home + "/include"
               
    client=" ECF_PASS=%ECF_PASS% ECF_NAME=%ECF_NAME% ecflow_client "
    hosts = ["cca", "ccb", "cct", "ecgb", "lxc", "opensuse131", "localhost"]
    suite = ecf.Suite(sname).add(
        Defstatus("suspended"),
        Variables(ECF_JOB_CMD= submit + " %USER% %HOST% %ECF_JOB% %ECF_JOBOUT%",
                  ECF_KILL_CMD= rid + "kill",
                  ECF_STATUS_CMD= rid + "status",
                  ECF_CHECK_CMD= rid + "check",
                  USER = user,
                  ECF_HOME= ecf_home,
                  ECF_FILES= ecf_files,
                  ECF_INCLUDE= ecf_include,
                  ECF_EXTN= ".sms",
                  HOST="localhost", LOGDIR= "%ECF_HOME%", SMSOUT= "%ECF_HOME%", # trap.h
                  ),
        Family("limits").add(Defstatus("complete"),
                             Label("memo", "use click-mouse-3-Edit to leave a message"),
                             Label("begin", "Edit-Pref-Admin + suite-node-click3-Begin"),
                             Label("example", "four cases: no job, no submit - no job but submit - job created but no submit - job + submit"),
                             Limit("tasks", 10)),
        Inlimit("/%s/limits:tasks" % sname),
         
        Task("dummy").add(Variables(ECF_DUMMY_TASK= 1)),
        Family("ssh_login").add(
            Label("one_liner", "dummy script, no need for job, execute ECF_JOB_CMD"),
            Repeat("HOST", start=hosts, kind="enumerated"),
            Variables(ECF_JOB_CMD= client + "--init;" +
                      # "xterm -T %HOST% -e ssh %HOST%;" +
                      "ssh %HOST% pwd;" +
                      client + "--complete;"),
            Task("simple"),
        ),
        Family("preprocess").add(
            Task("preproc").add(
                Variables(ECF_JOB_CMD= client + " --init;"
                          + client + "--complete")),
            Task("simple").add(
                Label("info", ""),
                Event(2),
                Trigger("preproc eq complete")),
        ),
        Family("submit").add(
            Trigger(["ssh_login", ]),
            [ Family(host).add(
                Variables(HOST= host),
                Task("simple").add(Event(1),
                                   Meter("step", -1, 100, 90),
                                   Label("info", "")),
            ) for host in hosts ]
        ),
        Family("process").add(
            Label("info", "outer-follower-family + inner-run-sometime example"),
            Label("stop", "at first abort"),
            Trigger("./process ne aborted"),
            Family("daily").add(
                Repeat("YMD", 20160101, 20321212, kind="date"),
                Task("simple"),
                Family("decade").add(
                    Label("info", "Show-Icons-Complete"),
                    Complete("../daily:YMD % 10 ne 0"),
                    Task("simple")),
            ),
            Family("monthly").add(
                Trigger("monthly:YM lt daily:YMD / 100 or daily eq complete"),
                Repeat(name="YM", start=
                       [ "%d" % ( YM)
                         for YM in range(201601, 203212+1)
                         if (YM % 100) < 13],
                       kind="enum"),
                Task("simple"),
                Family("odd").add(
                    Complete("../monthly:YM % 2 eq 0"),
                    Task("simple")),
),
            Family("yearly").add(
                Repeat("Y", 2016, 2032, kind="integer"),
                Trigger("yearly:Y lt daily:YMD / 10000 or daily eq complete"),
                Task("simple"),
                Family("decade").add(
                    Complete("../yearly:Y % 10 ne 0"),
                    Task("simple")),
                Family("century").add(
                    Complete("../yearly:Y % 100 ne 0"),
                    Task("simple")),
            )
        )
    )
    # sh.mkdir(ecf_files);     sh.mkdir( ecf_include);     sh.ln("-sf ~emos/def/o/trap.h %s" % ecf_include)
    os.system(""
+ "mkdir %s;" % ecf_files
+ "mkdir %s;" % ecf_include
+ "ln -sf ~emos/def/o/include/trap.h %s;" % ecf_include # trap + init + lxop .running
              + "touch %s/rcp.h;" % ecf_include
              + "touch %s/env_setup.h;" % ecf_include
# +"ln -sf ~emos/def/o/include/env_setup.h %s;" % ecf_include # link PBS job to .running
# +"ln -sf ~emos/def/o/include/rcp.h %s;" % ecf_include # rapatriate remote job@completion
    )
    create_wrapper(ecf_files + "/simple.sms", task_content)
    create_wrapper(ecf_files + "/preproc.sms", inc_content)
    defs.add_suite(suite)
    print "#MSG: replacing", path, env["host"], env["port"]
    ecf.Client(env["host"], env["port"]).replace(path, defs, 1, 1)
class Seed(object):
    """ find min/max meter value below a node """
    def __init__(self, host=None, port=None, path=None, task=None):
        import signal
        if host is None:
            host = os.getenv("ECF_NODE", "$ECF_NODE$")
        if port is None and not MICRO[0] in host:
            try: port = int(os.getenv("ECF_PORT", "$ECF_PORT$"))
            except: port = 31415
        if host and port:
            self.clt = ec.Client(host, port)
        else: self.clt = None
        self.cl2 = None
        if path is None:
            if "$SUITE$" != "/admin" and not "SUITE" in "$SUITE$":
                self.path = ( "/$SUITE$/$FAMILY$/%s" % legs[0],
                              "/$SUITE$/$FAMILY$/%s" % legs[1],
                          )
            else: self.path = MC_STEPS_LIST
        else: self.path = (path, )
        if task is None:
            self.task = TARGET_TASK
        else: self.task = task
        # shall make sense when processed into job by ecflow
        # name remains when not processed...
        ecfv = {"ECF_NODE": "$ECF_NODE$",
                "ECF_PASS": "$ECF_PASS$",
                "ECF_NAME": "$ECF_NAME$",
                "ECF_PORT": "$ECF_PORT$",
                "ECF_TRYNO": "$ECF_TRYNO$",        }
        for key, val in ecfv.items():
            if key in val:
                ecfv[key] = os.getenv(key, None)
        if ecfv["ECF_NODE"] and ecfv["ECF_NAME"]:
            print "#MSG will communicate with server..."
            print "#kill: ssh %s kill -15 %d" % (ecfv["ECF_NODE"], os.getpid())
            self.cl2 = ec.Client()
            self.cl2.set_host_port(ecfv["ECF_NODE"], ecfv["ECF_PORT"])
            self.cl2.set_child_pid(os.getpid())
            self.cl2.set_child_path(ecfv["ECF_NAME"])
            self.cl2.set_child_password(ecfv["ECF_PASS"])
            self.cl2.set_child_try_no(int(ecfv["ECF_TRYNO"]))
            self.cl2.child_init()
            self.cl2.set_child_timeout(20)
            for sig in (signal.SIGINT,
                        signal.SIGHUP,
                        signal.SIGQUIT,
                        signal.SIGILL,
                        signal.SIGTRAP,
                        signal.SIGIOT,
                        signal.SIGBUS,
                        signal.SIGFPE,
                        signal.SIGUSR1,
                        signal.SIGUSR2,
                        signal.SIGPIPE,
                        signal.SIGTERM,
                        signal.SIGXCPU,
                        signal.SIGPWR):
                signal.signal(sig, self.signal_handler)
    def signal_handler(self, signum, frame):
        """ catch signal """
        print 'Aborting: Signal handler called with signal ', signum
        self.cl2.child_abort("Signal handler called with signal " + str(signum))
    def report(self, msg, meter=None):
        """ communicate with ecFlow server """
        if not self.cl2:
            return
        if meter:
            self.cl2.child_meter(msg, meter)
        elif msg == "stop":
            self.cl2.child_complete()
            sys.exit(0)
        else: self.cl2.child_label("info", msg)
def usage():
    """ help """
    print """client
  -o: operational node is to be replaced, default is test node,
      provide this option BEFORE -r
  -r: replace task node
  -h: this help
ECF_NODE=localhost ECF_PORT=31415 ./ecflow_suite.py -s start
"""
if __name__ == '__main__':
    try:
        OPTS, ARGS = getopt.getopt(
            sys.argv[1:], "hros:t:n:p:",
            ["help", "replace", "oper", "task", "suite", "node", "port"])
    except getopt.GetoptError as err:
        print "# what?", usage()
        sys.exit(2)
    TEST = 1
    PATH = None
    TASK = None
    SUITE = "start"
    PORT = None
    NODE = None
    for o, a in OPTS:
        if o in ("-r", "--replace", ):
            loader(TEST, host=NODE, port=PORT, path=TASK)
            sys.exit(0)
        # elif o in ("-p", "--path", ): PATH = a
        elif o in ("-o", "--oper", ):
            TEST = 0
        elif o in ("-t", "--task", ):
            TASK = a
        elif o in ("-s", "--suite", ): SUITE = a
        elif o in ("-n", "--node", ): NODE = a
        elif o in ("-p", "--port", ): PORT = a
"""
python ecflow_suite.py -s suite -p 1630 -n eurus -r
"""


#!/usr/bin/env python2.7
#////////1/////////2/////////3/////////4/////////5/////////6/////////7/////////8
# Name        :
# Author      : Avi
# Revision    : $Revision: #10 $
#
# Copyright 2009- ECMWF.
# This software is licensed under the terms of the Apache Licence version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
# In applying this licence, ECMWF does not waive the privileges and immunities
# granted to it by virtue of its status as an intergovernmental organisation
# nor does it submit to any jurisdiction.
#////////1/////////2/////////3/////////4/////////5/////////6/////////7/////////8
"""Check if expected modules are loaded.

Usage:
 ./check_modules.py --m "module load git module swap PrgEnv-cray PrgEnv-gnu " --f module_list_file
 ./check_modules.py --m "module swap PrgEnv-cray PrgEnv-gnu" --f module_list_file
 ./check_modules.py --m "module unload grib_api" --f module_list_file
 ./check_modules.py --m "module load gnu/4.8.1 module load python/2.7.8-01" --f module_list_file
 ./check_modules.py --m "module swap PrgEnv-cray PrgEnv-gnu module load cdt/16.03" --f module_list_file
 ./check_modules.py --m "module swap craype-network-aries craype-network-none module switch fftw/3.3.4.7" --f module_list_file

module_list_file is the output of module list e.g.

Currently Loaded Modulefiles:
  1) version/3.2.10(default)             7) ecfs/2.2.1-rc2(new:prodn:default)
  2) verbose/true(default)               8) metview/4.6.4(default)
  3) mode/64(default)                    9) ecaccess/4.0.2(default)
  4) gnu/4.8.1(default)                 10) fftw/3.3.4(default)
  5) python/2.7.8-01(default)           11) emos/437(default)
  6) sms/4.4.14(default)                12) ecmwf/1.0(default)
"""

import argparse
import re
import sys

def main():
    PARSER = argparse.ArgumentParser(description=__doc__, formatter_class=argparse.RawDescriptionHelpFormatter)
    PARSER.add_argument('--m', help="Space separated list of module commands")
    PARSER.add_argument('--f', help="Path to a file that is the output of module list")
    PARSER.add_argument('--remote_host', help="remote host")
    ARGS = PARSER.parse_args()

    delimter = "module "
    module_cmds = [e.strip() for e in ARGS.m.split(delimter) if e.strip() != ""]
    print("module_cmds ",module_cmds)

    module_actions = ("load", "add", "rm", "unload", "swap", "switch")
    modules_required = dict()
    modules_missing = set()

    def set_version(s):
        print(" set_version ",s)
        try:
            mod, version = s.split('/')
        except ValueError:
            mod = s
            version = None
        modules_required[mod] = version
        if mod in modules_missing:
             modules_missing.remove(mod)

    def unset_version(s):
        # Unloading a specific version has no effect if a different version is loaded!
        print(" unset_version ",s)
        try:
            mod, version = s.split('/')
            if mod in modules_required and modules_required[mod] == version:
                modules_required.pop(mod)
            modules_missing.add(s)
        except ValueError:
            if s in modules_required:
                modules_required.pop(s)
            modules_missing.add(s)

    print("=========================================================")
    print("Read input module list->",module_cmds)
    print("=========================================================")
    for mod in module_cmds:
        module_cmd = mod.split()
        print("module_cmd ",module_cmd)
        action = module_cmd[0]
        if action not in module_actions:
            print("Error: Module command ", module_cmd, " is not correctly formed, second arg must be one of", ', '.join(module_actions))
            return 1
        if action in ('load', 'add'):
            if len(module_cmd) != 2:
                print("Error: invalid module command '%s'" % ' '.join(module_cmd))
            set_version(module_cmd[1])
        if action in ('swap', 'switch'):
            # Swap takes either 1 or 2 arguments
            # module swap pkg/version <-- unloads pkg, loads pkg/version
            # module swap pkg1/v1 pkg2/v2 <-- unloads pkg1/v1, loads pkg2/v2
            if len(module_cmd) > 3:
                print("Error: invalid module command '%s'" % ' '.join(module_cmd))
            if len(module_cmd) == 3:
                unset_version(module_cmd[1])
            set_version(module_cmd[-1])
        if action in ('rm', 'unload'):
            if len(module_cmd) != 2:
                print("Error: invalid module command '%s'" % ' '.join(module_cmd))
            unset_version(module_cmd[1])

    print("=========================================================")
    print("Read module list file->",ARGS.f)
    print("=========================================================")
    modules_loaded = dict()
    module_list = list()
    with open(ARGS.f) as module_list_file:
        for line in module_list_file:
            #print line
            line = line.rstrip()
            if line.find("Currently Loaded") == 0: continue;
            tokens = re.split(r'\s|\d+\)',line)
            while '' in tokens: tokens.remove('')
            #print "tokens",tokens
            for tok in tokens:
                #print "tok",tok
                count = tok.count('/')
                if count == 1:
                   mod,version = tok.split('/')
                   version = version.split("(")[0]
                   print("->(",mod,",",version,")")
                   modules_loaded[mod] = version
                else:
                   print("->(",tok,",None)")
                   modules_loaded[tok] = None

    print("=========================================================")
    print("Checking")
    print("=========================================================")
    print("->module_missing:   ",modules_missing)
    print("->modules_required: ",modules_required)
    print("->module list:      ",modules_loaded)
    failed = 0
    for mod, version in list(modules_required.items()):
        if version and modules_loaded.get(mod) != version or version is None and mod not in modules_loaded:
            print("Error: expected module %s/%s not found in module list" % (mod, version))
            failed = 1
    for module in modules_missing:
        print("modules_missing - ", module)
        if '/' in module:
            mod, version = module.split('/')
            if modules_loaded.get(mod) == version:
                print("Error: unexpected module %s/%s found in module list" % (mod, version))
                failed = 1
        else:
            if module in modules_loaded:
                print("Error: unexpected module %s found in module list" % module)
                failed = 1
    return failed

if __name__ == "__main__":
    sys.exit(main())



import os
import ecf
from ecf import *

home = os.getenv("HOME") + "/ecflow_server"
user = os.getenv("USER")
port = os.getenv("ECF_PORT")
node = Suite("lorenz").add(
    Defstatus("suspended"),
    ecf.Edit(
        ECF_HOME=home,
        ECF_INCLUDE=home + "/include",
        ECF_FILES=home + "/files",
        ECF_OUT=home,
        ECF_EXTN=".ecf",
        USER=user,
        SCHOST="localhost",
        ECF_JOB_CMD="/home/ma/emos/bin/trimurti %USER% %SCHOST% %ECF_JOB% %ECF_JOBOUT%",
    ),
    ecf.Family("ecf").add(
        Task("compute").add(),
        ecf.Family("multi").add(  # once imported, alter script, run multiple tasks
            [
                ecf.Family("%02d" % num).add(
                    Edit(XYZ="[%d.0, %d.0, %d.0]" % (num, num, num)), Task("compute")
                )
                for num in range(0, 5)
            ]
        ),
    ),
)
client = ecf.Client("localhost@%s" % port)
defs = ecf.Defs()
defs.add_suite(node)
client.replace("/lorenz", defs)



#
# Copyright 2009- ECMWF.
#
# This software is licensed under the terms of the Apache Licence version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
# In applying this licence, ECMWF does not waive the privileges and immunities
# granted to it by virtue of its status as an intergovernmental organisation
# nor does it submit to any jurisdiction.
#

import os
import io
import ecflow as ecf
import itertools as it

import ecflow_test_util as Test


def to_str(defs):
    buffer = io.StringIO()
    print(defs, file=buffer)
    return buffer.getvalue()


def can_create_aviso_from_parameters():
    aviso = ecf.AvisoAttr("name", "listener", "url", "schema", "polling", "auth")
    assert aviso.name() == "name"
    assert aviso.listener() == "'listener'"
    assert aviso.url() == "url"
    assert aviso.schema() == "schema"
    assert aviso.polling() == "polling"
    assert aviso.auth() == "auth"


def can_create_aviso_from_default_parameters_0():
    suite = ecf.Suite("s1")

    family = ecf.Family("f1")
    suite.add_family(family)

    task = ecf.Task("f1", ecf.AvisoAttr("name", "listener"))
    assert len(list(task.avisos)) == 1

    actual = list(task.avisos)[0]
    assert actual.name() == "name"
    assert actual.listener() == "'listener'"
    assert actual.url() == "%ECF_AVISO_URL%"
    assert actual.schema() == "%ECF_AVISO_SCHEMA%"
    assert actual.polling() == "%ECF_AVISO_POLLING%"
    assert actual.auth() == "%ECF_AVISO_AUTH%"


def can_create_aviso_from_default_parameters_1():
    suite = ecf.Suite("s1")

    family = ecf.Family("f1")
    suite.add_family(family)

    task = ecf.Task("f1", ecf.AvisoAttr("name", "listener", "url"))
    assert len(list(task.avisos)) == 1

    actual = list(task.avisos)[0]
    assert actual.name() == "name"
    assert actual.listener() == "'listener'"
    assert actual.url() == "url"
    assert actual.schema() == "%ECF_AVISO_SCHEMA%"
    assert actual.polling() == "%ECF_AVISO_POLLING%"
    assert actual.auth() == "%ECF_AVISO_AUTH%"


def can_create_aviso_from_default_parameters_2():
    suite = ecf.Suite("s1")

    family = ecf.Family("f1")
    suite.add_family(family)

    task = ecf.Task("f1", ecf.AvisoAttr("name", "listener", "url", "schema"))
    assert len(list(task.avisos)) == 1

    actual = list(task.avisos)[0]
    assert actual.name() == "name"
    assert actual.listener() == "'listener'"
    assert actual.url() == "url"
    assert actual.schema() == "schema"
    assert actual.polling() == "%ECF_AVISO_POLLING%"
    assert actual.auth() == "%ECF_AVISO_AUTH%"


def can_create_aviso_from_default_parameters_3():
    suite = ecf.Suite("s1")

    family = ecf.Family("f1")
    suite.add_family(family)

    task = ecf.Task("f1", ecf.AvisoAttr("name", "listener", "url", "schema", "polling"))
    assert len(list(task.avisos)) == 1

    actual = list(task.avisos)[0]
    assert actual.name() == "name"
    assert actual.listener() == "'listener'"
    assert actual.url() == "url"
    assert actual.schema() == "schema"
    assert actual.polling() == "polling"
    assert actual.auth() == "%ECF_AVISO_AUTH%"


def can_create_aviso_with_listener_details():
    defs = ecf.Defs()
    suite = defs.add_suite("s")
    family = ecf.Family("f")
    suite.add_family(family)
    task = ecf.Task('t',
                    ecf.AvisoAttr('aviso',
                                  '{ "event": "dissemination", "request": { "destination": "CL1", "class": "od", "expver": "1", "stream": "oper", "step": [0, 12] } }',
                                  'https://aviso.ecmwf.int',
                                  'schema.json',
                                  '60',
                                  '/.ecmwfapirc'))
    family.add_task(task)

    content = to_str(defs)

    assert f"aviso" in content
    assert f'--listener \'{{ "event": "dissemination", "request": {{ "destination": "CL1", "class": "od", "expver": "1", "stream": "oper", "step": [0, 12] }} }}\'' in content
    assert f"--url https://aviso.ecmwf.int" in content
    assert f"--schema schema.json" in content
    assert f"--polling 60" in content
    assert f"--auth /.ecmwfapirc" in content


def can_add_aviso_to_task():
    suite = ecf.Suite("s1")

    family = ecf.Family("f1")
    suite.add_family(family)

    task = ecf.Task("f1")
    family.add_task(task)

    aviso = ecf.AvisoAttr("name", "listener", "url", "schema", "polling", "auth")
    task.add_aviso(aviso)
    assert len(list(task.avisos)) == 1

    actual = list(task.avisos)[0]
    assert actual.name() == "name"
    assert actual.listener() == "'listener'"
    assert actual.url() == "url"
    assert actual.schema() == "schema"
    assert actual.polling() == "polling"
    assert actual.auth() == "auth"


def can_embed_aviso_into_task():
    suite = ecf.Suite("s1")

    family = ecf.Family("f1")
    suite.add_family(family)

    task = ecf.Task("f1", ecf.AvisoAttr("name", "listener", "url", "schema", "polling", "auth"))
    assert len(list(task.avisos)) == 1

    actual = list(task.avisos)[0]
    assert actual.name() == "name"
    assert actual.listener() == "'listener'"
    assert actual.url() == "url"
    assert actual.schema() == "schema"
    assert actual.polling() == "polling"
    assert actual.auth() == "auth"


def cannot_have_multiple_avisos_in_single_task():
    suite = ecf.Suite("s1")

    family = ecf.Family("f1")
    suite.add_family(family)

    try:
        task = ecf.Task("f1", ecf.AvisoAttr("name", "listener", "url", "schema", "polling", "auth"),
                        ecf.AvisoAttr("another", "listener", "url", "schema", "polling", "auth"))
        assert False, "Expected exception indicating multiple avisos are not allowed in a task"
    except RuntimeError as e:
        assert True


def can_check_job_creation_with_aviso():
    defs = ecf.Defs()

    suite = ecf.Suite("s")
    defs.add_suite(suite)

    family = ecf.Family("f")
    suite.add_family(family)

    task = ecf.Task("t")
    family.add_task(task)

    aviso = ecf.AvisoAttr("name", "listener", "url", "schema", "polling", "auth")
    task.add_aviso(aviso)

    defs.check_job_creation()


if __name__ == "__main__":
    Test.print_test_start(os.path.basename(__file__))

    can_create_aviso_from_parameters()
    can_create_aviso_from_default_parameters_0()
    can_create_aviso_from_default_parameters_1()
    can_create_aviso_from_default_parameters_2()
    can_create_aviso_from_default_parameters_3()
    can_create_aviso_with_listener_details()
    can_add_aviso_to_task()
    can_embed_aviso_into_task()
    cannot_have_multiple_avisos_in_single_task()
    can_check_job_creation_with_aviso()

    print("All tests pass")



#////////1/////////2/////////3/////////4/////////5/////////6/////////7/////////8
# Name        :
# Author      : Avi
# Revision    : $Revision: #10 $
#
# Copyright 2009- ECMWF.
# This software is licensed under the terms of the Apache Licence version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
# In applying this licence, ECMWF does not waive the privileges and immunities
# granted to it by virtue of its status as an intergovernmental organisation
# nor does it submit to any jurisdiction.
#////////1/////////2/////////3/////////4/////////5/////////6/////////7/////////8
from ecflow import Alias, AttrType, Autocancel, CheckPt, ChildCmdType, Client, Clock, Cron, DState, Date, Day, Days, \
                  Defs, Ecf, Event, Expression, Family, FamilyVec, File, Flag, FlagType, FlagTypeVec, InLimit, \
                  JobCreationCtrl, Label, Late, Limit, Meter, Node, NodeContainer, NodeVec, PartExpression, PrintStyle, \
                  Repeat, RepeatDate, RepeatDateTime, RepeatDateList, RepeatDay, RepeatEnumerated, RepeatInteger, \
                  RepeatString, SState, State, Style, Submittable, Suite, SuiteVec, Task, TaskVec, Time, TimeSeries, \
                  TimeSlot, Today, UrlCmd, Variable, VariableList, Verify, WhyCmd, ZombieAttr, ZombieType, \
                  ZombieUserActionType, Trigger, Complete, Edit, Defstatus
import unittest 
import sys

class Test_dunder_rshift(unittest.TestCase):
    def test_node_dunder_rshift(self):
        # will ONLY work if we have starting NodeContainer
        suite = Suite('s') >> Task('t1') >> Task('t2') >> Task('t3') >> Task('t4')
        self.assertEqual(len(list(suite)),4,"expected 4 children but found " + str(len(list(suite))) )
        
        self.assertEqual(str(suite.t2.get_trigger()),"t1 == complete","Trigger not as expected: " + str( suite.t2.get_trigger())) 
        self.assertEqual(str(suite.t3.get_trigger()),"t2 == complete","Trigger not as expected: " + str( suite.t3.get_trigger())) 
        self.assertEqual(str(suite.t4.get_trigger()),"t3 == complete","Trigger not as expected: " + str( suite.t4.get_trigger())) 

        fam = Family("f1") >> Task('t1') >> Task('t2') >> Task('t3') >> Task('t4')
        self.assertEqual(len(list(fam)),4,"expected 4 children but found " + str(len(list(fam))) )

        self.assertEqual(str(fam.t2.get_trigger()),"t1 == complete","Trigger not as expected: " + str( fam.t2.get_trigger())) 
        self.assertEqual(str(fam.t3.get_trigger()),"t2 == complete","Trigger not as expected: " + str( fam.t3.get_trigger())) 
        self.assertEqual(str(fam.t4.get_trigger()),"t3 == complete","Trigger not as expected: " + str( fam.t4.get_trigger())) 

    def test_node_dunder_rshift_trigger_cat(self):
        suite = Suite('s')
        # will ONLY work if we have starting NodeContainer
        suite >> Task('t1') >> (Task('t2') + Trigger("x == 1")) >> (Task('t3') + Trigger("y==1")) >> Task('t4')
        self.assertEqual(len(list(suite)),4,"expected 4 children but found " + str(len(list(suite))) )
        
        self.assertEqual(str(suite.t2.get_trigger()),"x == 1 AND t1 == complete","Trigger not as expected: " + str( suite.t2.get_trigger())) 
        self.assertEqual(str(suite.t3.get_trigger()),"y==1 AND t2 == complete","Trigger not as expected: " + str( suite.t3.get_trigger())) 
        self.assertEqual(str(suite.t4.get_trigger()),"t3 == complete","Trigger not as expected: " + str( suite.t4.get_trigger())) 

class Test_dunder_lshift(unittest.TestCase):
    def test_node_dunder_lshift(self):
        # will ONLY work if we have starting NodeContainer
        suite = Suite('s') << Task('t1') << Task('t2') << Task('t3') << Task('t4')
        self.assertEqual(len(list(suite)),4,"expected 4 children but found " + str(len(list(suite))) )
        
        self.assertEqual(suite.t4.get_trigger(),None,"Trigger not as expected: " + str( suite.t4.get_trigger())) 
        self.assertEqual(str(suite.t3.get_trigger()),"t4 == complete","Trigger not as expected: " + str( suite.t3.get_trigger())) 
        self.assertEqual(str(suite.t2.get_trigger()),"t3 == complete","Trigger not as expected: " + str( suite.t2.get_trigger())) 
        self.assertEqual(str(suite.t1.get_trigger()),"t2 == complete","Trigger not as expected: " + str( suite.t1.get_trigger())) 
 
        fam = Family("f1") << Task('t1') << Task('t2') << Task('t3') << Task('t4')
        self.assertEqual(len(list(fam)),4,"expected 4 children but found " + str(len(list(fam))) )
 
        self.assertEqual(fam.t4.get_trigger(),None,"Trigger not as expected: " + str( fam.t4.get_trigger())) 
        self.assertEqual(str(fam.t3.get_trigger()),"t4 == complete","Trigger not as expected: " + str( fam.t3.get_trigger())) 
        self.assertEqual(str(fam.t2.get_trigger()),"t3 == complete","Trigger not as expected: " + str( fam.t2.get_trigger())) 
        self.assertEqual(str(fam.t1.get_trigger()),"t2 == complete","Trigger not as expected: " + str( fam.t1.get_trigger())) 

    def test_node_dunder_lshift_trigger_cat(self):
        suite = Suite('s')
        # will ONLY work if we have starting NodeContainer
        suite >> Task('t1') << (Task('t2') + Trigger("x == 1")) << (Task('t3') + Trigger("y==1")) << Task('t4')
        self.assertEqual(len(list(suite)),4,"expected 4 children but found " + str(len(list(suite))) )
         
        self.assertEqual(suite.t4.get_trigger(),None,"Trigger not as expected: " + str( suite.t4.get_trigger())) 
        self.assertEqual(str(suite.t3.get_trigger()),"y==1 AND t4 == complete","Trigger not as expected: " + str( suite.t3.get_trigger())) 
        self.assertEqual(str(suite.t2.get_trigger()),"x == 1 AND t3 == complete","Trigger not as expected: " + str( suite.t2.get_trigger())) 
        self.assertEqual(str(suite.t1.get_trigger()),"t2 == complete","Trigger not as expected: " + str( suite.t1.get_trigger())) 

class Test_dunder_add(unittest.TestCase):
    
    def test_node_dunder_add_variable(self):
        # will ONLY work if we have starting NodeContainer
        suite = Suite("s") + Family("f") + Family("f2") + Task("t3") + Edit(name="value")
        self.assertEqual(len(list(suite.variables)),1 ,"expected suite to have 1 variable " + str(len(list(suite.variables))) )

        suite = Suite("s") + Family("f") + Family("f2") + (Task("t3") + Edit(name="value"))
        self.assertEqual(len(list(suite.t3.variables)),1 ,"expected task t3 to have 1 variable " + str(len(list(suite.t3.variables))) )
                
    def test_defs_dunder_add(self):
        # will ONLY work if we have starting defs
        defs = Defs() + Suite("s") + Suite("s1")
        self.assertEqual(len(defs),2 ,"expected 2 suites but found " + str(len(defs)) )
        
        defs += Suite("sx")  #   + Suite('x')  wont work,   Suite('sx').__add__(Suite('x') not defined
        self.assertEqual(len(defs),3 ,"expected 3 suites but found " + str(len(defs)) )
        
        defs + Suite("a") + Suite("b")
        self.assertEqual(len(defs),5 ,"expected 5 suites but found " + str(len(defs)) )

    def test_node_dunder_add_meter(self):
        # will ONLY work if we have starting NodeContainer
        suite = Suite("s") + Family("f") + Family("f2") + Task("t3") + Edit(name="value")
        suite.t3 + Event(11,"event") + Meter("meter",0,10,10) + Label("label","c") + Trigger("1==1") 
        self.assertEqual(len(suite),3 ,"expected 3 children but found " + str(len(suite)) )
        self.assertEqual(len(list(suite.t3.events)),1 ,"expected 1 event found " + str(len(list(suite.t3.events))) )
        self.assertEqual(len(list(suite.t3.meters)),1 ,"expected 1 event found " + str(len(list(suite.t3.meters))) )
        self.assertEqual(len(list(suite.t3.labels)),1 ,"expected 1 event found " + str(len(list(suite.t3.labels))) )

    def test_node_dunder_add(self):
        suite = Suite("s")  
        suite += Task("t3")  # + Task("t2") not allowed as Task("t1").__add__("t2") not defined
        suite.t3 += Event(11,"event")  # + Task("t2") not allowed as Task("t1").__add__("t2") not defined
        self.assertEqual(len(suite),1 ,"expected 1 children but found " + str(len(suite)) )
        self.assertEqual(len(list(suite.t3.events)),1 ,"expected 1 event found " + str(len(list(suite.t3.events))) )

        suite = Suite('s') + [ Task("t{0}".format(i)) for i in range(1,5)]
        self.assertEqual(len(suite),4 ,"expected 4 children but found " + str(len(suite)) )

    def test_dunder_add_all(self):
        defs = Defs() + Suite("s1")
        defs.s1 += Clock(1, 1, 2010, False)
        defs.s1 += Autocancel(1, 10, True)
        defs.s1 += Task('t1') + Edit({ "a":"y", "b":"bb"}, c="v",d="b") + Edit({ "e":1, "f":"bb"}) +  Edit(g="d") + Edit(h=1) +\
                    Event(1) + Event(11,"event") + Meter("meter",0,10,10) + Label("label","c") + Trigger("1==1") + \
                    Complete("1==1") + Limit("limit",10) + Limit("limit2",10) + InLimit("limitName","/limit",2) + \
                    Defstatus(DState.complete) + Today(0,30) + Today("00:59") + Today("00:00 11:30 00:01") + \
                    Time(0,30) + Time("00:59") + Time("00:00 11:30 00:01") + Day("sunday") + Day(Days.monday) + \
                    Date(1,1,0) + Date(28,2,1960) + Autocancel(3) 
                    
        t1 = defs.find_abs_node("/s1/t1")
        self.assertTrue(t1 is not None, "Can't find t1")
        self.assertEqual(len(defs.s1),1, "Expected 1 nodes but found " + str(len(defs.s1)))
        self.assertEqual(len(list(t1.variables)), 8, "expected 8 variables but found " + str(len(list(t1.variables))))
        self.assertEqual(len(list(t1.limits)), 2, "expected 2 limits")
        self.assertEqual(len(list(t1.inlimits)), 1, "expected 1 inlimits")
        self.assertEqual(len(list(t1.events)), 2, "expected 2 events")
        self.assertEqual(len(list(t1.meters)), 1, "expected 1 meter")
        self.assertEqual(len(list(t1.labels)), 1, "expected 1 label")
        self.assertEqual(len(list(t1.times)), 3, "expected 3 times")
        self.assertEqual(len(list(t1.todays)), 3, "expected 3 times")
        self.assertEqual(len(list(t1.days)), 2, "expected 2 days")
        self.assertEqual(len(list(t1.dates)), 2, "expected 2 dates")
        self.assertTrue(t1.get_trigger(), "Can't find t1 trigger")
        self.assertTrue(t1.get_complete(), "Can't find t1 complete")
        self.assertTrue(t1.get_autocancel(), "Can't find t1 autocancel")
        #print(defs)


class Test_crash(unittest.TestCase):
    def test_trigger_node_list(self):
        suite = Suite("s")
        fam = suite.add_family("f")
        task = fam.add_task("t")
        self.assertEqual(task.get_abs_node_path(),"/s/f/t","Path not as expected " + task.get_abs_node_path() )
 
        suite = Suite("s")
        fam = Family("f")
        task = Task("t")
        suite.add_family(fam)
        task = fam.add_task(task)
        self.assertEqual(task.get_abs_node_path(),"/s/f/t","Path not as expected " + task.get_abs_node_path() )

        defs = Defs()
        task = defs.add_suite("s").add_family("f").add_task("t")
        self.assertEqual(task.get_abs_node_path(),"/s/f/t","Path not as expected " + task.get_abs_node_path() )

        # The following will crash or cause memory error. Since the Defs() is transitory/temp()
        # Unlike the previous example where we keep the defs in a variable.
        #task = Defs().add_suite("s").add_family("f").add_task("t")
        #self.assertEqual(task.get_abs_node_path(),"/s/f/t","Path not as expected " + task.get_abs_node_path() )
         
        # Like wise this will crash since the Suite() is transitory reference counted, and goes out of scope.
#         task = Suite("s").add_family("f").add_task("t")
#         self.assertEqual(task.get_abs_node_path(),"/s/f/t","Path not as expected " + task.get_abs_node_path() )

class TestListComprehension(unittest.TestCase):
    def test_suite_list(self):
        defs = Defs()
        defs += [ Suite("s{0}".format(i)) for i in range(1,6) ]
        self.assertEqual(len(defs), 5, " expected 5 suites but found " + str(len(defs)))

        defs.add( [ Suite("s{0}".format(i)) for i in range(6,11) ] )
        self.assertEqual(len(defs), 10, " expected 10 suites but found " + str(len(defs)))

        defs = Defs( [ Suite("s{0}".format(i)) for i in range(1,6) ] )
        self.assertEqual(len(defs), 5, " expected 5 suites but found " + str(len(defs)))

    def test_family_list(self):
        defs = Defs()
        defs += [ Suite("suite").add( [ Family("f{0}".format(i)) for i in range(1,6)]  ) ]
        self.assertEqual(len(defs.suite), 5, " expected 5 familes but found " + str(len(defs.suite)))

        defs = Defs( [ Suite("xx", [ Family("f{0}".format(i)) for i in range(1,6)]  ) ] )
        self.assertEqual(len(defs.xx), 5, " expected 5 familes but found " + str(len(defs.xx)))

    def test_task_list(self):
        defs = Defs()
        defs += [ Suite("suite").add( Family("f").add( [ Task("t{0}".format(i)) for i in range(1,6)]  )) ]
        self.assertEqual(len(defs.suite.f), 5, " expected 5 task but found " + str(len(defs.suite.f)))

        defs = Defs( [ Suite("suite", Family("f",[ Task("t{0}".format(i)) for i in range(1,6)]  )) ] )
        self.assertEqual(len(defs.suite.f), 5, " expected 5 task but found " + str(len(defs.suite.f)))

    def test_task_list2(self):
        defs = Defs()
        defs += [ Suite("suite").add( Task("x"), 
                                      Family("f").add( [ Task("t{0}".format(i)) for i in range(1,6)]  ),
                                      Task("y"),
                                      [ Family("f{0}".format(i)) for i in range(1,6) ],
                                      Edit(a="b"),
                                      [ Task("t{0}".format(i)) for i in range(1,6) ],
                                    )
                ]
        self.assertEqual(len(defs.suite), 13, " expected 13 nodes but found " + str(len(defs.suite)))
        self.assertEqual(len(defs.suite.f), 5, " expected 5 nodes but found " + str(len(defs.suite.f)))
        self.assertEqual(len(list(defs.suite.variables)), 1, " expected 1 variable " + str(len(list(defs.suite.variables))))

    def test_5Suite_with_5families_with_5tasks(self):
        defs = Defs().add(   
                [ Suite("s{0}".format(i)).add( 
                    [ Family("f{0}".format(i)).add( 
                        [ Task("t{0}".format(i)) 
                              for i in range(1,6)] ) 
                        for i in range(1,6)]  ) 
                    for i in range(1,6) ] )
        #print defs
        self.assertEqual(len(defs), 5, " expected 5 suites but found " + str(len(defs)))
        for suite in defs:
            self.assertEqual(len(suite), 5, " expected 5 familes but found " + str(len(suite)))
            for fam in suite:
                self.assertEqual(len(fam), 5, " expected 5 tasks but found " + str(len(fam)))
                
    def test__5Suite_with_5families_with_5tasks(self):
        defs = Defs()
        defs += [ Suite("s{0}".format(i)).add( 
                        [ Family("f{0}".format(i)).add( 
                            [ Task("t{0}".format(i)) for i in range(1,6)] ) 
                        for i in range(1,6)] ) 
                    for i in range(1,6) 
                ]
        #print defs
        self.assertEqual(len(defs), 5, " expected 5 suites but found " + str(len(defs)))
        for suite in defs:
            self.assertEqual(len(suite), 5, " expected 5 familes but found " + str(len(suite)))
            for fam in suite:
                self.assertEqual(len(fam), 5, " expected 5 tasks but found " + str(len(fam)))
        

class TestTrigger(unittest.TestCase):
    def test_trigger(self):
        t = Trigger("a == complete")
        self.assertEqual(str(t),"a == complete","Trigger not as expected:"+ str(t))

    def test_trigger_string_list(self):
        t = Trigger(["a"])
        self.assertEqual(str(t),"a == complete","Trigger not as expected: "+ str(t))

        t = Trigger(["a == 1"])
        self.assertEqual(str(t),"a == 1","Trigger not as expected: "+ str(t))
        
        t = Trigger(["a","b","c"])
        self.assertEqual(str(t),"a == complete AND b == complete AND c == complete","Trigger not as expected: " + str(t))

    def test_trigger_node_list(self):
        t = Trigger([Task("a")])
        self.assertEqual(str(t),"a == complete","Trigger not as expected: "+ str(t))

        defs = Defs()
        task = defs.add_suite("s").add_family("f").add_task("t")
        self.assertEqual(task.get_abs_node_path(),"/s/f/t","Path not as expected " + task.get_abs_node_path() )
        t = Trigger([task])
        self.assertEqual(str(t),"/s/f/t == complete","Trigger not as expected: "+ str(t))

    def test_trigger_string_and_node_list(self):
        defs = Defs()
        task = defs.add_suite("s").add_family("f").add_task("t")
        t = Trigger(["a","b",task])
        self.assertEqual(str(t),"a == complete AND b == complete AND /s/f/t == complete","Trigger not as expected: " + str(t))
        task += t
        self.assertEqual(str(task.get_trigger() ),"a == complete AND b == complete AND /s/f/t == complete","Trigger not as expected: " + str(task.get_trigger())) 

    def test_add(self):
        defs = Defs()
        task = defs.add_suite("s").add_family("f").add_task("t")
        task.add(Trigger("1==1"))
        expr = task.get_trigger()
        self.assertEqual(str(expr),"1==1","Trigger not as expected: " +  expr.get_expression())
        
        task.add(Trigger("2==1"))
        self.assertEqual(str(expr),"1==1 AND 2==1","Trigger not as expected: " +  str(expr))
        
    def test_add_composition(self):
        defs = Defs()
        task = defs.add_suite("s").add_family("f").add_task("t")
        task.add(Trigger("1==1"))
        task.add(Trigger("1==2",False))
        task.add(Trigger("x==2"))
        expr = task.get_trigger()
        self.assertEqual(str(expr),"1==1 OR 1==2 AND x==2","Trigger not as expected: " +  expr.get_expression())


class TestComplete(unittest.TestCase):
    def test_trigger(self):
        t = Complete("a == complete")
        self.assertEqual(str(t),"a == complete","Complete not as expected:"+ str(t))

    def test_Complete_string_list(self):
        t = Complete(["a"])
        self.assertEqual(str(t),"a == complete","Complete not as expected: "+ str(t))
        
        t = Complete(["a","b","c"])
        self.assertEqual(str(t),"a == complete AND b == complete AND c == complete","Complete not as expected: " + str(t))

    def test_Complete_node_list(self):
        t = Complete([Task("a")])
        self.assertEqual(str(t),"a == complete","Complete not as expected: "+ str(t))

        defs = Defs()
        task = defs.add_suite("s").add_family("f").add_task("t")
        self.assertEqual(task.get_abs_node_path(),"/s/f/t","Path not as expected " + task.get_abs_node_path() )
        t = Complete([task])
        self.assertEqual(str(t),"/s/f/t == complete","Complete not as expected: "+ str(t))

    def test_Complete_string_and_node_list(self):
        defs = Defs()
        task = defs.add_suite("s").add_family("f").add_task("t")
        t = Complete(["a","b",task])
        self.assertEqual(str(t),"a == complete AND b == complete AND /s/f/t == complete","Complete not as expected: " + str(t))

    def test_add(self):
        defs = Defs()
        task = defs.add_suite("s").add_family("f").add_task("t")
        task.add(Complete("1==1"))
        expr = task.get_complete()
        self.assertEqual(str(expr),"1==1","Complete not as expected: " +  expr.get_expression())
        
        task.add(Complete("2==1"))
        self.assertEqual(str(expr),"1==1 AND 2==1","Complete not as expected: " +  str(expr))
        
    def test_add_composition(self):
        defs = Defs()
        task = defs.add_suite("s").add_family("f").add_task("t")
        task.add( Complete("1==1"))
        task.add( Complete("1==2",False))
        task.add( Complete("x==2"))
        expr = task.get_complete()
        self.assertEqual(str(expr),"1==1 OR 1==2 AND x==2","Trigger not as expected: " +  expr.get_expression())


class TestDefsAdd(unittest.TestCase):
    def test_add_suite1(self):
        defs = Defs().add(Suite("a"))
        self.assertEqual(len(defs),1,"Expected 1 suite to be added")

    def test_add_suite3(self):
        defs = Defs().add(Suite("a"),Suite("b"),Suite("c").add(Edit(a="b")))
        self.assertEqual(len(defs),3,"Expected 3 suite to be added")

    def test_add_variables(self):
        defs = Defs().add(Edit(a="b",b='c',d='e'))
        self.assertEqual(len(list(defs.user_variables)),3,"Expected 3 variables to be added")

    def test_add_variables2(self):
        defs = Defs(Edit(a="b",b='c',d='e'))
        self.assertEqual(len(list(defs.user_variables)),3,"Expected 3 variables to be added")
        
class TestSuiteAdd(unittest.TestCase):
    def test_add_suite1(self):
        s = Suite("a").add(Family("a").add(Edit(a="b")))
        self.assertEqual(len(s),1,"Expected 1 child to be added")

class TestFamilyAdd(unittest.TestCase):
    def test_add_task(self):
        s = Family("a").add(Task("a").add(Edit(a="b")))
        self.assertEqual(len(s),1,"Expected 1 child to be added")

class TestDefstatus(unittest.TestCase):
    def test_illegal_defstatus(self):
        if sys.version_info[0] == 2 and sys.version_info[1] >= 7:
            with self.assertRaises(RuntimeError):
                Defstatus("fred")
            
class TestEdit(unittest.TestCase):
    def test_add_variable(self):
        defs = Defs().add(Suite("s").add(Edit(a=1,b=1),Edit(c="c")))
        self.assertEqual(len(list(defs.s.variables)), 3, "expected 3 variables")
    
    def test_edit(self):
        t = Task("t1").add(
                    Edit({ "a":"y", "b":"bb"}, c="v",d="b"),
                    Edit({ "e":"1", "f":"bb"}),
                    Edit(g="d"),
                    Edit(h="1"))
        self.assertEqual(len(list(t.variables)), 8, "expected 8 variables but found " + str(len(list(t.variables))))
        
class TestAddAll(unittest.TestCase):
    def test_add(self):
        defs = Defs().add(
            Suite("s1").add(
                Clock(1, 1, 2010, False),
                Autocancel(1, 10, True),
                Task("t1").add(
                    Edit({ "a":"y", "b":"bb"}, c="v",d="b"),
                    Edit({ "e":1, "f":"bb"}),
                    Edit(g="d"),
                    Edit(h=1),
                    Event(1),
                    Event(11,"event"),
                    Meter("meter",0,10,10),
                    Label("label","c"),
                    Trigger("1==1"),
                    Complete("1==1"),
                    Limit("limit",10),Limit("limit2",10),
                    InLimit("limitName","/limit",2),
                    Defstatus(DState.complete),
                    Today(0,30),Today("00:59"),Today("00:00 11:30 00:01"),
                    Time(0,30),Time("00:59"),Time("00:00 11:30 00:01"),
                    Day("sunday"),Day(Days.monday),
                    Date(1,1,0),Date(28,2,1960),
                    Autocancel(3)
                    ),
                [ Family("f{0}".format(i)) for i in range(1,6)]
            )
        )
        t1 = defs.find_abs_node("/s1/t1")
        self.assertTrue(t1 is not None, "Can't find t1")
        self.assertEqual(len(defs.s1),6, "Expected 6 nodes but found " + str(len(defs.s1)))
        self.assertEqual(len(list(t1.variables)), 8, "expected 8 variables")
        self.assertEqual(len(list(t1.limits)), 2, "expected 2 limits")
        self.assertEqual(len(list(t1.inlimits)), 1, "expected 1 inlimits")
        self.assertEqual(len(list(t1.events)), 2, "expected 2 events")
        self.assertEqual(len(list(t1.meters)), 1, "expected 1 meter")
        self.assertEqual(len(list(t1.labels)), 1, "expected 1 label")
        self.assertEqual(len(list(t1.times)), 3, "expected 3 times")
        self.assertEqual(len(list(t1.todays)), 3, "expected 3 times")
        self.assertEqual(len(list(t1.days)), 2, "expected 2 days")
        self.assertEqual(len(list(t1.dates)), 2, "expected 2 dates")
        self.assertTrue(t1.get_trigger(), "Can't find t1 trigger")
        self.assertTrue(t1.get_complete(), "Can't find t1 complete")
        self.assertTrue(t1.get_autocancel(), "Can't find t1 autocancel")
        #print(defs)

    def test_raw_constructor(self):
        defs = Defs().add(
            Suite("s1", 
                Clock(1, 1, 2010, False), 
                Autocancel(1, 10, True),
                Task("t1",
                    Edit({ "a":"y", "b":"bb"}, c="v",d="b"),
                    Edit({ "e":1, "f":"bb"}),
                    Edit(g="d"),
                    Edit(h=1),
                    Event(1),
                    Event(11,"event"),
                    Meter("meter",0,10,10),
                    Label("label","c"),
                    Trigger("1==1"),
                    Complete("1==1"),
                    Limit("limit",10),Limit("limit2",10),
                    InLimit("limitName","/limit",2),
                    Defstatus(DState.complete),
                    Today(0,30),Today("00:59"),Today("00:00 11:30 00:01"),
                    Time(0,30),Time("00:59"),Time("00:00 11:30 00:01"),
                    Day("sunday"),Day(Days.monday),
                    Date(1,1,0),Date(28,2,1960),
                    Autocancel(3)
                    ),
                [ Family("f{0}".format(i)) for i in range(1,6)]
            )
        )
        print(defs)
        t1 = defs.find_abs_node("/s1/t1")
        self.assertTrue(t1 is not None, "Can't find t1")
        self.assertEqual(len(defs.s1),6, "Expected 6 nodes but found " + str(len(defs.s1)))
        self.assertEqual(len(list(t1.variables)), 8, "expected 8 variables")
        self.assertEqual(len(list(t1.limits)), 2, "expected 2 limits")
        self.assertEqual(len(list(t1.inlimits)), 1, "expected 1 inlimits")
        self.assertEqual(len(list(t1.events)), 2, "expected 2 events")
        self.assertEqual(len(list(t1.meters)), 1, "expected 1 meter")
        self.assertEqual(len(list(t1.labels)), 1, "expected 1 label")
        self.assertEqual(len(list(t1.times)), 3, "expected 3 times")
        self.assertEqual(len(list(t1.todays)), 3, "expected 3 times")
        self.assertEqual(len(list(t1.days)), 2, "expected 2 days")
        self.assertEqual(len(list(t1.dates)), 2, "expected 2 dates")
        self.assertTrue(t1.get_trigger(), "Can't find t1 trigger")
        self.assertTrue(t1.get_complete(), "Can't find t1 complete")
        self.assertTrue(t1.get_autocancel(), "Can't find t1 autocancel")
     
     
class TestIAdd(unittest.TestCase):
    def test_iadd_with_getattr(self):
        defs = Defs();
        s1 = defs.add_suite("s1")
        s1 += [ Task("a"),Task("b")]
        defs.s1.a += [ Edit({ "x1":"y", "aa1":"bb"}, a="v",b="b",),
                    Edit({ "x":"y", "aa":"bb"}),
                    Edit(d="d")
                ]
        defs.s1.b += [ Event(1),
                    Event(11,"event"),
                    Meter("meter",0,10,10),
                    Label("label","c"),
                ]
        self.assertEqual(len(list(defs.s1.a.variables)), 7, "expected 7 variables")
        self.assertEqual(len(list(defs.s1.b.events)), 2, "expected 2 events")
        self.assertEqual(len(list(defs.s1.b.meters)), 1, "expected 1 meter")
        self.assertEqual(len(list(defs.s1.b.labels)), 1, "expected 1 label")

    def test_iadd(self):
        defs = Defs();
        defs += [ Suite("s2"),Edit({ "x1":"y", "aa1":"bb"}, a="v",b="b",) ]
        s1 = defs.add_suite("s1")
        s1 += [ Clock(1, 1, 2010, False), Autocancel(1, 10, True) ] #+ returns the same node back
        t1 = s1.add_task("t1")
        t1 +=   [   Edit({ "x1":"y", "aa1":"bb"}, a="v",b="b",),
                    Edit({ "x":"y", "aa":"bb"}),
                    Edit(d="d"),
                    Event(1),
                    Event(11,"event"),
                    Meter("meter",0,10,10),
                    Label("label","c"),
                    Trigger("1==1"),
                    Complete("1==1"),
                    Limit("limit",10),Limit("limit2",10),
                    InLimit("limitName","/limit",2),
                    Defstatus(DState.complete),
                    Today(0,30),Today("00:59"),Today("00:00 11:30 00:01"),
                    Time(0,30),Time("00:59"),Time("00:00 11:30 00:01"),
                    Day("sunday"),Day(Days.monday),
                    Date(1,1,0),Date(28,2,1960),
                    Autocancel(3)
                ]
        self.assertTrue(t1 is not None, "Can't find t1")
        self.assertEqual(len(list(defs.suites)), 2, "expected 7 suites")
        self.assertEqual(len(list(t1.variables)), 7, "expected 7 variables")
        self.assertEqual(len(list(t1.limits)), 2, "expected 2 limits")
        self.assertEqual(len(list(t1.inlimits)), 1, "expected 1 inlimits")
        self.assertEqual(len(list(t1.events)), 2, "expected 2 events")
        self.assertEqual(len(list(t1.meters)), 1, "expected 1 meter")
        self.assertEqual(len(list(t1.labels)), 1, "expected 1 label")
        self.assertEqual(len(list(t1.times)), 3, "expected 3 times")
        self.assertEqual(len(list(t1.todays)), 3, "expected 3 times")
        self.assertEqual(len(list(t1.days)), 2, "expected 2 days")
        self.assertEqual(len(list(t1.dates)), 2, "expected 2 dates")
        self.assertTrue(t1.get_trigger(), "Can't find t1 trigger")
        self.assertTrue(t1.get_complete(), "Can't find t1 complete")
        self.assertTrue(t1.get_autocancel(), "Can't find t1 autocancel")
        #print(defs)
        
class TestComparison(unittest.TestCase):
    def setUp(self):
        self.cron = Cron()
        self.cron.set_time_series(TimeSeries(TimeSlot(23 , 0) ,True)) # True means relative to suite start
        self.late = Late()
        self.late.submitted(TimeSlot(20, 10))
        self.late.active(TimeSlot(20, 10))
        self.late.complete(TimeSlot(20, 10), True)
        self.late2 = Late()
        self.late2.submitted(20, 10)
        self.late2.active(20, 10)
        self.late2.complete(20, 10, True)
    
        self.defs1 = Defs()
        self.defs1.add_suite("s1").add_task("t1").add_variable("a","v").add_event(1).add_event(11,"event").add_meter("meter",0,10,10).add_label("label","c")
        self.defs1.add_suite("s2").add_family("f1").add_task("t1").add_trigger("1==1").add_complete("1==1")
        self.defs1.add_suite("s3").add_family("f1").add_family("f2").add_task("t1").add_variable("var","v")
        self.defs1.add_suite("edit").add_variable("a","a").add_variable("b","b") 
        self.defs1.add_suite("limit").add_limit("limit",10).add_limit("limit2",10) 
        self.defs1.add_suite("inlimit").add_inlimit("limitName", "/limit", 2)  
        self.defs1.add_suite("RepeatInteger").add_repeat(RepeatInteger("integer", 0, 100, 2)) 
        self.defs1.add_suite("RepeatEnumerated").add_repeat(RepeatEnumerated("enum", ["red", "green", "blue" ])) 
        self.defs1.add_suite("RepeatDate").add_repeat(RepeatDate("ymd", 20100111, 20100115, 2))
        self.defs1.add_suite("RepeatDateTime").add_repeat(RepeatDateTime("ymd", "20100111T000000", "20100115T000000", "48:00:00"))
        self.defs1.add_suite("RepeatDateList").add_repeat(RepeatDateList("ymd",[20100111, 20100115]))
        self.defs1.add_suite("RepeatString").add_repeat(RepeatString("string", ["a", "b", "c" ]))
        self.defs1.add_suite("RepeatDay").add_repeat(RepeatDay(1))
        self.defs1.add_suite("defstatus").add_defstatus(DState.active) 
        self.defs1.add_suite("today").add_task("today").add_today("00:30").add_today(0,59).add_today("00:00 11:30 00:01")
        self.defs1.add_suite("time").add_task("time").add_time("00:30").add_time(0,59).add_time("00:00 11:30 00:01")
        self.defs1.add_suite("day").add_task("day").add_day("sunday").add_day(Days.monday)
        self.defs1.add_suite("date").add_task("date").add_date(1,1,0).add_date(28,2,1960)
        self.defs1.add_suite("cron").add_task("cron").add_cron(self.cron)
        self.defs1.add_suite("late").add_family("late").add_late(self.late).add_task("late").add_late(self.late2)
        self.defs1.add_suite("clock").add_clock(Clock(1, 1, 2010, False))
        self.defs1.add_suite("autocancel").add_autocancel(3)
        
        zombie_suite = self.defs1.add_suite("zombie") 
        self.zombie_life_time_in_server = 800
        self.child_list = [ ChildCmdType.init, ChildCmdType.event, ChildCmdType.meter, ChildCmdType.label, ChildCmdType.wait, ChildCmdType.abort,ChildCmdType.complete ]
        zombie_type_list = [ ZombieType.ecf, ZombieType.user,ZombieType.path ]
        for zombie_type in zombie_type_list:
            zombie_attr = ZombieAttr(zombie_type, self.child_list, ZombieUserActionType.block, self.zombie_life_time_in_server)
            zombie_suite.add_zombie(zombie_attr)
   
 
    def test_compare_with_add(self):
        defs = Defs().add(
            Suite("s1").add(
                Task("t1").add(
                    Edit(a="v"),
                    Event(1),
                    Event(11,"event"),
                    Meter("meter",0,10,10),
                    Label("label","c")
                )
            ),
            Suite("s2").add(
                Family("f1").add(
                    Task("t1").add(
                        Trigger("1==1"),
                        Complete("1==1")
                    )
                )
            ),
            Suite("s3").add(
                Family("f1").add(
                    Family("f2").add(
                        Task("t1").add(
                            Edit(var = "v")
                        )
                    )
                )
            ),
            Suite("edit").add( Edit({"a": "a"}), Edit({ "b" : "b"}) ), # add separate, otherwise with dict order is undefined
            Suite("limit").add( Limit("limit",10),Limit("limit2",10)),
            Suite("inlimit").add( InLimit("limitName","/limit",2)),
            Suite("RepeatInteger").add( RepeatInteger("integer", 0, 100, 2)),
            Suite("RepeatEnumerated").add( RepeatEnumerated("enum", ["red", "green", "blue" ])),
            Suite("RepeatDate").add( RepeatDate("ymd", 20100111, 20100115, 2)),
            Suite("RepeatDateTime").add( RepeatDateTime("ymd", "20100111T000000", "20100115T000000", "48:00:00")),
            Suite("RepeatDateList").add( RepeatDateList("ymd", [20100111, 20100115])),
            Suite("RepeatString").add( RepeatString("string", ["a", "b", "c" ])),
            Suite("RepeatDay").add( RepeatDay(1)),
            Suite("defstatus").add( Defstatus(DState.active)),
            Suite("today").add(
                Task("today").add(
                    Today(0,30),Today("00:59"),Today("00:00 11:30 00:01")
                )
            ),
            Suite("time").add(
                Task("time").add(
                    Time(0,30),Time("00:59"),Time("00:00 11:30 00:01")
                )
            ),
            Suite("day").add(
                Task("day").add(
                    Day("sunday"),Day(Days.monday)
                )
            ),
            Suite("date").add(
                Task("date").add(
                    Date(1,1,0),Date(28,2,1960)
                )
            ),
            Suite("cron").add(
                Task("cron").add(
                    self.cron
                )
            ),
            Suite("late").add(
                Family("late").add(
                    self.late,
                    Task("late").add(
                        self.late2
                    )
                )
            ),
            Suite("clock").add( Clock(1, 1, 2010, False)),
            Suite("autocancel").add( Autocancel(3)),
            Suite("zombie").add( 
                ZombieAttr( ZombieType.ecf, self.child_list, ZombieUserActionType.block, self.zombie_life_time_in_server),
                ZombieAttr( ZombieType.user, self.child_list, ZombieUserActionType.block, self.zombie_life_time_in_server),
                ZombieAttr( ZombieType.path, self.child_list, ZombieUserActionType.block, self.zombie_life_time_in_server)
            )
        )      

        self.assertEqual(self.defs1,defs,"defs not equal\n" + str(self.defs1) + "\n\n" + str(defs))    

    def test_compare_with_raw_constructor(self):
        defs = Defs()
        defs += Suite('s1',Task('t1',Event(1),Event(11,'event'),Meter("meter",0,10,10),Label("label","c"),a='v'))
        defs += Suite('s2',Family('f1',Task('t1',Trigger("1==1"),Complete("1==1"))))
        defs += Suite('s3',Family('f1',Family('f2',Task('t1',var='v'))))
        defs += Suite('edit',Edit(a='a'),Edit(b='b'))
        defs += Suite('limit',Limit("limit",10),Limit("limit2",10))
        defs += Suite('inlimit',InLimit("limitName", "/limit", 2 ))
        defs += Suite('RepeatInteger',RepeatInteger("integer", 0, 100, 2) )
        defs += Suite('RepeatEnumerated', RepeatEnumerated("enum", ["red", "green", "blue" ]))
        defs += Suite('RepeatDate', RepeatDate("ymd", 20100111, 20100115, 2))
        defs += Suite('RepeatDateTime', RepeatDateTime("ymd", "20100111T000000", "20100115T000000", "48:00:00"))
        defs += Suite('RepeatDateList', RepeatDateList("ymd", [20100111, 20100115]))
        defs += Suite('RepeatString',RepeatString("string", ["a", "b", "c" ]))
        defs += Suite('RepeatDay', RepeatDay(1))
        defs += Suite('defstatus', Defstatus('active'))
        defs += Suite('today',Task('today',Today("00:30"),Today(0,59),Today("00:00 11:30 00:01")))
        defs += Suite('time',Task('time',Time("00:30"),Time(0,59),Time("00:00 11:30 00:01")))
        defs += Suite('day',Task('day',Day("sunday"),Day(Days.monday)))
        defs += Suite('date',Task('date',Date(1,1,0),Date(28,2,1960)))
        defs += Suite('cron',Task('cron',self.cron))
        defs += Suite('late',Family('late',self.late,Task('late',self.late2)))
        defs += Suite('clock',Clock(1, 1, 2010, False) )
        defs += Suite('autocancel',Autocancel(3) )
        
        defs += Suite("zombie",ZombieAttr( ZombieType.ecf, self.child_list, ZombieUserActionType.block, self.zombie_life_time_in_server),
                               ZombieAttr( ZombieType.user, self.child_list, ZombieUserActionType.block, self.zombie_life_time_in_server),
                               ZombieAttr( ZombieType.path, self.child_list, ZombieUserActionType.block, self.zombie_life_time_in_server))
                               
        self.assertEqual(self.defs1,defs,"defs not equal\n" + str(self.defs1) + "\n\n" + str(defs))    

        
if __name__ == "__main__":
    unittest.main()
    print("All Tests pass")



#////////1/////////2/////////3/////////4/////////5/////////6/////////7/////////8
# Name        :
# Author      : Avi
# Revision    : $Revision: #10 $
#
# Copyright 2009- ECMWF.
# This software is licensed under the terms of the Apache Licence version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
# In applying this licence, ECMWF does not waive the privileges and immunities
# granted to it by virtue of its status as an intergovernmental organisation
# nor does it submit to any jurisdiction.
#////////1/////////2/////////3/////////4/////////5/////////6/////////7/////////8

# code for testing  generated variables

import os
import ecflow
import ecflow_test_util as Test

if __name__ == "__main__":
    Test.print_test_start(os.path.basename(__file__))
     
    #===========================================================================
    print("Defs: test generated variables")
    #===========================================================================
    defs = ecflow.Defs()
    suite = defs.add_suite("s1")
    family = suite.add_family("f1")
    task = family.add_task("t1")

    print("\nsuite generated variables, Most of date related suite generated variables will only have values after the suite has begun")
    variable_list = ecflow.VariableList()
    suite.get_generated_variables(variable_list)
    for gen_var in variable_list:
        print(gen_var)
        gen_var = suite.find_gen_variable(gen_var.name())
        assert not gen_var.empty(), "Could not find suite generated variable " + gen_var
    assert len(list(variable_list)) == 14,"Expected 14 generated variables for suites"    
    assert variable_list[0].name() == "SUITE", "expected generated variable of name SUITE but found " + variable_list[0].name()
    assert variable_list[0].value() == "s1", "expected generated variable of value 's1' but found " + variable_list[0].value()


    print("\nfamily generated variables")
    variable_list = ecflow.VariableList()
    family.get_generated_variables(variable_list)
    for gen_var in variable_list:
        print(gen_var)
        gen_var = family.find_gen_variable(gen_var.name())
        assert not gen_var.empty(), "Could not find family generated variable " + gen_var

    assert len(list(variable_list)) == 2,"Expected 2 generated variables for families"    
    assert variable_list[0].name() == "FAMILY", "expected generated variable of name FAMILY but found " + variable_list[0].name()
    assert variable_list[0].value() == "f1", "expected generated variable of value 'f1' but found " + variable_list[0].value()
    assert variable_list[1].name() == "FAMILY1", "expected generated variable of name FAMILY1 but found " + variable_list[1].name()
    assert variable_list[1].value() == "f1", "expected generated variable of value 'f1' but found " + variable_list[1].value()

    print("\ntask generated variables")
    variable_list = ecflow.VariableList()
    task.get_generated_variables(variable_list)
    for gen_var in variable_list:
        print(gen_var)
        gen_var = task.find_gen_variable(gen_var.name())
        assert not gen_var.empty(), "Could not find task generated variable " + gen_var

    assert len(list(variable_list)) == 8,"Expected 8 generated variables for tasks"    
    assert variable_list[0].name() == "TASK", "expected generated variable of name TASK but found " + variable_list[0].name()
    assert variable_list[0].value() == "t1", "expected generated variable of value 't1' but found " + variable_list[0].value()
    assert variable_list[1].name() == "ECF_JOB", "expected generated variable of name ECF_JOB but found " + variable_list[1].name()
    assert variable_list[1].value() == "./s1/f1/t1.job0", "expected generated variable of value './s1/f1/t1.job0' but found " + variable_list[1].value()
    assert variable_list[2].name() == "ECF_SCRIPT", "expected generated variable of name ECF_SCRIPT but found " + variable_list[2].name()
    assert variable_list[2].value() == "./s1/f1/t1.ecf", "expected generated variable of value './s1/f1/t1.ecf' but found " + variable_list[2].value()
    assert variable_list[3].name() == "ECF_JOBOUT", "expected generated variable of name ECF_JOBOUT but found " + variable_list[3].name()
    assert variable_list[3].value() == "./s1/f1/t1.0", "expected generated variable of value './s1/f1/t1.0' but found " + variable_list[3].value()
    assert variable_list[4].name() == "ECF_TRYNO", "expected generated variable of name ECF_TRYNO but found " + variable_list[4].name()
    assert variable_list[4].value() == "0", "expected generated variable of value '0' but found " + variable_list[4].value()
    assert variable_list[5].name() == "ECF_RID", "expected generated variable of name ECF_RID but found " + variable_list[5].name()
    assert variable_list[5].value() == "", "expected generated variable of value '' but found " + variable_list[5].value()
    assert variable_list[6].name() == "ECF_NAME", "expected generated variable of name ECF_NAME but found " + variable_list[6].name()
    assert variable_list[6].value() == "/s1/f1/t1", "expected generated variable of value '/s1/f1/t1' but found " + variable_list[6].value()
    assert variable_list[7].name() == "ECF_PASS", "expected generated variable of name ECF_NAME but found " + variable_list[7].name()
    assert variable_list[7].value() == "", "expected generated variable of value '' but found " + variable_list[7].value()

    print("\nAll Tests pass")
    
